@@ -9,6 +9,7 @@
 
 #include "aptitude.h"
 
+#include <vscreen/fragment.h>
 #include <vscreen/vs_button.h>
 #include <vscreen/vs_center.h>
 #include <vscreen/vs_frame.h>
@@ -18,17 +19,20 @@
 #include <vscreen/config/keybindings.h>
 #include <vscreen/config/colors.h>
 
-#include <generic/apt.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
 
-#include <sigc++/slot.h>
-#include <sigc++/object_slot.h>
-#include <sigc++/bind.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/functors/mem_fun.h>
+
+#include <string>
+#include <vector>
 
 using namespace std;
 
 struct option_item
 {
-  enum {OPTION_BOOL, OPTION_STRING, OPTION_END} type;
+  enum {OPTION_BOOL, OPTION_STRING, OPTION_RADIO, OPTION_END} type;
   const char *description;
   const char *option_name;
 
@@ -38,6 +42,13 @@ struct option_item
     const char *s_default;
   };
 
+  /** Used by radio options: the valid values for this option. */
+  vector<string> choices;
+
+  /** Used by radio options: the descriptions corresponding to
+      elements of "choices". */
+  vector<string> choice_descriptions;
+
   option_item()
     :type(OPTION_END), description(NULL), option_name(NULL)
   {
@@ -56,6 +67,43 @@ struct option_item
      s_default(def)
   {
   }
+
+  static option_item radio(const char *description,
+			   const char *option_name,
+			   const char *def,
+			   ...)
+  {
+    option_item rval;
+
+    va_list args;
+    va_start(args, def);
+
+    rval.type=OPTION_RADIO;
+    rval.description=description;
+    rval.option_name=option_name;
+    rval.s_default=def;
+
+    while(1)
+      {
+	const char *choice=va_arg(args, const char *);
+
+	if(choice == NULL)
+	  break;
+
+	const char *description=va_arg(args, const char *);
+
+	if(description == NULL)
+	  break;
+
+	rval.choices.push_back(choice);
+
+	rval.choice_descriptions.push_back(description);
+      }
+
+    va_end(args);
+
+    return rval;
+  }
 };
 
 option_item ui_options[]={
@@ -71,8 +119,12 @@ option_item ui_options[]={
 	      PACKAGE "::UI::Exit-On-Last-Close", true),
   option_item(N_("Prompt for confirmation at exit"),
 	      PACKAGE "::UI::Prompt-On-Exit", true),
-  option_item(N_("Pause after downloading files"),
-	      PACKAGE "::UI::Pause-After-Download", true),
+  option_item::radio(N_("Pause after downloading files"),
+		     PACKAGE "::UI::Pause-After-Download", "OnlyIfError",
+		     "No", _("Never"),
+		     "OnlyIfError", _("When an error occurs"),
+		     "Yes", _("Always"),
+		     NULL, NULL),
   option_item(N_("Use a 'status-line' download indicator for all downloads"),
 	      PACKAGE "::UI::Minibuf-Download-Bar", false),
   option_item(N_("Display the extended description area by default"),
@@ -116,6 +168,9 @@ option_item misc_options[]={
   option_item(N_("Forget which packages are \"new\" whenever packages are installed or removed"),
 	      PACKAGE "::Forget-New-On-Install",
 	      false),
+  option_item(N_("Do not display a warning when the first change is made in read-only mode."),
+	      PACKAGE "::Suppress-Read-Only-Warning",
+	      false),
   option_item(N_("Warn when attempting to perform a privileged action as a non-root user"),
 	      PACKAGE "::Warn-Not-Root",
 	      true),
@@ -134,17 +189,26 @@ option_item dependency_options[]={
 	      PACKAGE "::Recommends-Important", true),
   option_item(N_("Remove unused packages automatically"),
 	      PACKAGE "::Delete-Unused", true),
-  option_item(N_("Automatically remove unused packages matching this filter"),
-	      PACKAGE "::Delete-Unused-Pattern", ""),
+  option_item(N_("Don't automatically remove unused packages matching this filter"),
+	      PACKAGE "::Keep-Unused-Pattern", ""),
   option_item()
 };
 
 // Commits all the entries of an options dialog, then saves the
 // present settings and destroys the dialog.
-class dialog_manager:public SigC::Object
+class dialog_manager:public sigc::trackable
 {
 public:
-  dialog_manager(vscreen_widget *_dialog):dialog(_dialog) {}
+  dialog_manager(const vs_widget_ref &_dialog):dialog(_dialog) {}
+
+  ~dialog_manager()
+  {
+    for(vector<apt_config_widget *>::iterator i=widgets.begin();
+	i!=widgets.end(); ++i)
+      delete *i;
+
+    widgets.clear();
+  }
 
   void add_widget(apt_config_widget *w) {widgets.push_back(w);}
 
@@ -169,41 +233,34 @@ public:
 private:
   vector<apt_config_widget *> widgets;
 
-  vscreen_widget *dialog;
+  vs_widget_ref dialog;
 };
 
-static vscreen_widget *realize_options_dialog(option_item *items)
+static vs_widget_ref realize_options_dialog(option_item *items)
 {
-  vs_center *center=new vs_center;
+  vs_center_ref center=vs_center::create();
 
   // Create general infrastructure (FIXME: make a plugin dialog widget thingy
   // that does this?
-  vs_table *table=new vs_table;
+  vs_table_ref table=vs_table::create();
   table->set_colsep(1);
-  table->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
 
-  vs_button *okbutton=new vs_button(_("Ok"));
-  vs_button *cancelbutton=new vs_button(_("Cancel"));
-  okbutton->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
-  cancelbutton->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
+  vs_button_ref okbutton=vs_button::create(_("Ok"));
+  vs_button_ref cancelbutton=vs_button::create(_("Cancel"));
 
-  table->connect_key("Confirm", &global_bindings, okbutton->pressed.slot());
-  table->connect_key("Cancel", &global_bindings, cancelbutton->pressed.slot());
+  table->connect_key("Confirm", &global_bindings, okbutton->pressed.make_slot());
+  table->connect_key("Cancel", &global_bindings, cancelbutton->pressed.make_slot());
 
   dialog_manager *manager=new dialog_manager(center);
 
-  okbutton->pressed.connect(slot(*manager, &dialog_manager::commit));
-  cancelbutton->pressed.connect(slot(*manager, &dialog_manager::destroy));
-
-  // FIXME: tables should support left/center/right-alignment..
-  vs_center *cok=new vs_center(okbutton);
-  vs_center *ccancel=new vs_center(cancelbutton);
+  okbutton->pressed.connect(sigc::mem_fun(*manager, &dialog_manager::commit));
+  cancelbutton->pressed.connect(sigc::mem_fun(*manager, &dialog_manager::destroy));
 
   int row=0;
 
   while(items->type!=option_item::OPTION_END)
     {
-      vs_label *l=NULL;
+      vs_label_ref l=NULL;
 
       switch(items->type)
 	{
@@ -213,57 +270,117 @@ static vscreen_widget *realize_options_dialog(option_item *items)
 						   items->option_name,
 						   items->b_default);
 
-	    w->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
-
-	    table->add_widget(w, row, 0, 1, 2, true, false);
+	    table->add_widget_opts(w->cb, row, 0, 1, 2,
+				   vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
+				   vs_table::EXPAND);
 
 	    manager->add_widget(w);
 	  }
+	  ++row;
 	  break;
 	case option_item::OPTION_STRING:
 	  {
 	    apt_string_widget *w=new apt_string_widget(items->option_name,
 						       items->s_default);
 
-	    l=new vs_label(_(items->description),
-			   get_color("DefaultWidgetBackground")|A_REVERSE);
+	    l = vs_label::create(flowbox(text_fragment(_(items->description))));
 
-	    table->add_widget(l, row, 0, 1, 1, false, false);
-	    table->add_widget(w, row, 1, 1, 1, true, false);
+	    table->add_widget_opts(l, row, 0, 1, 1,
+			      vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
+			      vs_table::EXPAND);
+	    table->add_widget_opts(w->w, row, 1, 1, 1,
+				   vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
+				   vs_table::EXPAND);
 
 	    manager->add_widget(w);
 	  }
+	  ++row;
 	  break;
+	case option_item::OPTION_RADIO:
+	  {
+	    assert(items->choices.size()==items->choice_descriptions.size());
+
+	    assert(items->choices.size()>0);
+
+	    apt_radio_widget *w=new apt_radio_widget(items->option_name,
+						     items->choices,
+						     items->s_default);
+
+	    string curr=aptcfg->Find(items->option_name,
+				     items->s_default);
+
+	    l = vs_label::create(flowbox(text_fragment(_(items->description))));
+
+	    for(vector<string>::size_type i=0; i<items->choices.size(); ++i)
+	      {
+		string choice=items->choices[i];
+		string description=items->choice_descriptions[i];
+
+		vs_togglebutton_ref b=vs_radiobutton::create(_(description.c_str()),
+							     choice == curr);
+
+		table->add_widget(b, row+i, 1, 1, 1,
+				  vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
+				  vs_table::EXPAND);
+
+		w->rg.add_button(b, i);
+	      }
+
+	    if(!w->rg.selection_valid())
+	      {
+		// Hm, if it didn't work the first time, why would it
+		// work the second time?
+		for(vector<string>::size_type i=0; i<items->choices.size(); ++i)
+		  if(items->choices[i]==curr)
+		    {
+		      w->rg.select(i);
+		      break;
+		    }
+	      }
+
+	    table->add_widget(l, row+(items->choices.size()-1)/2, 0, 1, 1, false, false);
+	    row+=items->choices.size();
+	    manager->add_widget(w);
+	  }
+	  break;
+
 	default:
 	  assert(0);  // Someone probably goofed in setting the tables up
 	}
 
       items++;
-      row++;
     }
 
-  table->add_widget(cok, row, 0, 1, 1, false, false);
-  table->add_widget(ccancel, row, 1, 1, 1, false, false);
-
-  vs_frame *frame=new vs_frame(table);
-  frame->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
+  vs_table_ref bt=vs_table::create();
+  bt->add_widget_opts(okbutton, 0, 0, 1, 1,
+		      vs_table::ALIGN_CENTER | vs_table::EXPAND,
+		      vs_table::ALIGN_CENTER);
+  bt->add_widget_opts(cancelbutton, 0, 1, 1, 1,
+		      vs_table::ALIGN_CENTER | vs_table::EXPAND,
+		      vs_table::ALIGN_CENTER);
+  table->add_widget_opts(bt, row, 0, 1, 2,
+			 vs_table::ALIGN_CENTER | vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
+			 vs_table::ALIGN_CENTER);
+  
+  vs_frame_ref frame=vs_frame::create(table);
 
   center->add_widget(frame);
+  center->set_bg_style(style_attrs_flip(A_REVERSE));
 
   return center;
 }
 
-vscreen_widget *make_ui_options_dialog()
+vs_widget_ref make_ui_options_dialog()
 {
   return realize_options_dialog(ui_options);
 }
 
-vscreen_widget *make_misc_options_dialog()
+vs_widget_ref make_misc_options_dialog()
 {
   return realize_options_dialog(misc_options);
 }
 
-vscreen_widget *make_dependency_options_dialog()
+vs_widget_ref make_dependency_options_dialog()
 {
   return realize_options_dialog(dependency_options);
 }
