@@ -8,21 +8,44 @@
 #ifndef APTITUDE_H
 #define APTITUDE_H
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 /* Take care of NLS matters.  */
 
-#if HAVE_LOCALE_H
+#ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
-#if !HAVE_SETLOCALE
-# define setlocale(Category, Locale) /* empty */
+#ifndef HAVE_SETLOCALE
+inline void setlocale(int, const char *)
+{
+}
 #endif
 
 #if ENABLE_NLS
 # include <libintl.h>
+# include <string.h>
 # define _(Text) gettext (Text)
 # define N_(Text) Text
+
+/** Strips everything up to and includeing the first pipe character
+ *  from the string.  Strings without a pipe character are unchanged.
+ */
+#ifdef __GNUG__
+__attribute__ ((format_arg(1)))
+#endif
+inline const char *P_(const char *Text)
+{
+  const char * const translation = gettext(Text);
+  const char * const stripto = strchr(translation, '|');
+
+  if(stripto == NULL)
+    return Text;
+  else
+    return stripto+1;
+}
+
 #else
 # undef bindtextdomain
 # define bindtextdomain(Domain, Directory) /* empty */
@@ -30,10 +53,12 @@
 # define textdomain(Domain) /* empty */
 # define _(Text) Text
 # define N_(Text) Text
+# define P_(Text) Text
 # define gettext(Text) Text
 # define dgettext(Domain, Text) Text
 #endif
 
+
 // This is used to store the location of the binary as determined by
 // argv[0].
 extern const char *argv0;
