@@ -7,16 +7,17 @@
 
 #include <aptitude.h>
 
-#include <generic/apt.h>
-#include <generic/matchers.h>
-#include <generic/tasks.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/matchers.h>
+#include <generic/apt/tasks.h>
 
 #include <apt-pkg/algorithms.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/pkgcache.h>
 
 bool cmdline_applyaction(cmdline_pkgaction_type action,
-			 pkgProblemResolver &fixer, pkgCache::PkgIterator pkg,
+			 pkgCache::PkgIterator pkg,
 			 pkgset &to_install, pkgset &to_hold,
 			 pkgset &to_remove, pkgset &to_purge,
 			 int verbose,
@@ -85,6 +86,7 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 
   switch(action)
     {
+    case cmdline_installauto:
     case cmdline_install:
       if(pkg.CurrentVer()!=ver || pkg->CurrentState!=pkgCache::State::Installed)
 	to_install.insert(pkg);
@@ -113,6 +115,12 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
     case cmdline_hold:
       to_hold.insert(pkg);
       break;
+    case cmdline_keep:
+      to_install.erase(pkg);
+      to_remove.erase(pkg);
+      to_purge.erase(pkg);
+      to_hold.erase(pkg);
+      break;
     case cmdline_unhold:
       to_hold.erase(pkg);
       break;
@@ -121,7 +129,7 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 	{
 	  if(pkg.CurrentVer().end())
 	    printf(_("Package %s is not installed, cannot forbid an upgrade\n"), pkg.Name());
-	  else if((*apt_cache_file)[pkg].Upgradable())
+	  else if(!(*apt_cache_file)[pkg].Upgradable())
 	    printf(_("Package %s is not upgradable, cannot forbid an upgrade\n"), pkg.Name());
 	}
     default:
@@ -130,40 +138,34 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 
   switch(action)
     {
+    case cmdline_installauto:
     case cmdline_install:
     case cmdline_reinstall:
       if(action==cmdline_reinstall && pkg.CurrentVer().end())
 	break;
 
       (*apt_cache_file)->set_candidate_version(ver, NULL);
-      (*apt_cache_file)->mark_install(pkg, true, action == cmdline_reinstall, NULL);
-      fixer.Clear(pkg);
-      fixer.Protect(pkg);
+      (*apt_cache_file)->mark_install(pkg, aptcfg->FindB(PACKAGE "::Auto-Install", true),
+				      action == cmdline_reinstall, NULL);
+      if(action == cmdline_installauto)
+	(*apt_cache_file)->mark_auto_installed(pkg, true, NULL);
       break;
     case cmdline_remove:
       (*apt_cache_file)->mark_delete(pkg, false, false, NULL);
-      fixer.Clear(pkg);
-      fixer.Protect(pkg);
-      fixer.Remove(pkg);
       break;
     case cmdline_purge:
       (*apt_cache_file)->mark_delete(pkg, true, false, NULL);
-      fixer.Clear(pkg);
-      fixer.Protect(pkg);
-      fixer.Remove(pkg);
       break;
     case cmdline_hold:
       (*apt_cache_file)->mark_keep(pkg, false, true, NULL);
-      fixer.Clear(pkg);
-      fixer.Protect(pkg);
+      break;
+    case cmdline_keep:
+      (*apt_cache_file)->mark_keep(pkg, false, false, NULL);
       break;
     case cmdline_unhold:
-      fixer.Clear(pkg);
       if(pkg->CurrentState==pkgCache::State::Installed)
-	{
-	  (*apt_cache_file)->mark_install(pkg, true, false, NULL);
-	  fixer.Protect(pkg);
-	}
+	(*apt_cache_file)->mark_install(pkg, aptcfg->FindB(PACKAGE "::Auto-Install", true),
+					false, NULL);
       else
 	(*apt_cache_file)->mark_keep(pkg, false, false, NULL);
       break;
@@ -194,7 +196,6 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 
 bool cmdline_applyaction(string s,
 			 cmdline_pkgaction_type action,
-			 pkgProblemResolver &fixer,
 			 pkgset &to_install, pkgset &to_hold,
 			 pkgset &to_remove, pkgset &to_purge,
 			 int verbose)
@@ -221,7 +222,7 @@ bool cmdline_applyaction(string s,
 	  for(std::list<std::string>::iterator i=tasks->begin();
 	      i!=tasks->end(); ++i)
 	    if(*i==s)
-	      rval=cmdline_applyaction(action, fixer, pkg,
+	      rval=cmdline_applyaction(action, pkg,
 				       to_install, to_hold, to_remove, to_purge,
 				       verbose, source,
 				       sourcestr) && rval;
@@ -235,13 +236,18 @@ bool cmdline_applyaction(string s,
     return false;
 
   // This is harmless for other commands, but it won't make sense.
-  if(source==cmdline_version_version && action!=cmdline_install && action!=cmdline_forbid_version)
+  if(source == cmdline_version_version &&
+     action != cmdline_install &&
+     action != cmdline_forbid_version &&
+     action != cmdline_installauto)
     {
-      printf(_("You can only specify a package version with an 'install' command.\n"));
+      printf(_("You can only specify a package version with an 'install' command or a 'forbid-version' command.\n"));
       return false;
     }
 
-  if(source==cmdline_version_archive && action!=cmdline_install)
+  if(source == cmdline_version_archive &&
+     action != cmdline_install &&
+     action != cmdline_installauto)
     {
       printf(_("You can only specify a package archive with an 'install' command.\n"));
       return false;
@@ -267,7 +273,7 @@ bool cmdline_applyaction(string s,
 	  for(pkgCache::PkgIterator j=(*apt_cache_file)->PkgBegin();
 	      !j.end(); ++j)
 	    {
-	      if(!(j.VersionList().end() && j.ProvidesList().end()) && m->matches(j, j.CurrentVer()))
+	      if(!(j.VersionList().end() && j.ProvidesList().end()) && m->matches(j))
 		possible.push_back(j);
 	    }
 
@@ -297,7 +303,7 @@ bool cmdline_applyaction(string s,
 	      for(pkgCache::PkgIterator j=(*apt_cache_file)->PkgBegin();
 		  !j.end(); ++j)
 		{
-		  if(m->matches(j, (*apt_cache_file)[j].CandidateVerIter(*apt_cache_file)))
+		  if(m->matches(j))
 		    possible.push_back(j);
 		}
 
@@ -317,7 +323,7 @@ bool cmdline_applyaction(string s,
 	  return false;
 	}
 
-      rval=cmdline_applyaction(action, fixer, pkg,
+      rval=cmdline_applyaction(action, pkg,
 			       to_install, to_hold, to_remove, to_purge,
 			       verbose, source,
 			       sourcestr);
@@ -336,16 +342,8 @@ bool cmdline_applyaction(string s,
 	{
 	  pkgCache::VerIterator testver;
 
-	  if(action==cmdline_install || action==cmdline_hold ||
-	     action==cmdline_forbid_version)
-	    testver=(*apt_cache_file)[pkg].CandidateVerIter(*apt_cache_file);
-	  else if(action==cmdline_remove || action==cmdline_purge)
-	    testver=pkg.CurrentVer();
-	  else
-	    testver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);
-
-	  if(m->matches(pkg, testver))
-	    rval=cmdline_applyaction(action, fixer, pkg,
+	  if(m->matches(pkg))
+	    rval=cmdline_applyaction(action, pkg,
 				     to_install, to_hold, to_remove, to_purge,
 				     verbose, source,
 				     sourcestr) && rval;
@@ -356,3 +354,109 @@ bool cmdline_applyaction(string s,
 
   return rval;
 }
+
+/** \return \b false iff the beginning of s doesn't look like an
+ *  action specifier.
+ */
+static bool parse_action_str(const string &s,
+			     string::size_type &loc,
+			     cmdline_pkgaction_type &action)
+{
+  if(loc<s.size())
+    {
+      switch(s[loc])
+	{
+	case '_':
+	  action=cmdline_purge;
+	  ++loc;
+	  break;
+	case '-':
+	  action=cmdline_remove;
+	  ++loc;
+	  break;
+	case '=':
+	  action=cmdline_hold;
+	  ++loc;
+	  break;
+	case '+':
+	  action=cmdline_install;
+	  ++loc;
+	  if(loc<s.size() && s[loc]=='M')
+	    {
+	      action=cmdline_installauto;
+	      ++loc;
+	    }
+	  break;
+	case ':':
+	  action=cmdline_keep;
+	  ++loc;
+	  break;
+	case '&':
+	  ++loc;
+	  if(loc<s.size())
+	    switch(s[loc])
+	      {
+	      case 'M':
+		action=cmdline_markauto;
+		++loc;
+		break;
+	      case 'm':
+		action=cmdline_unmarkauto;
+		++loc;
+		break;
+	      default:
+		return false;
+	      }
+	  break;
+	default:
+	  return false;
+	}
+      return true;
+    }
+  else
+    return false;
+}
+
+void cmdline_parse_action(string s,
+			  pkgset &to_install, pkgset &to_hold,
+			  pkgset &to_remove, pkgset &to_purge,
+			  int verbose)
+{
+  string::size_type loc=0;
+
+  while(loc<s.size() && isspace(s[loc]))
+    ++loc;
+
+  if(loc==s.size())
+    return;
+
+  cmdline_pkgaction_type action;
+
+  if(!parse_action_str(s, loc, action))
+    {
+      printf(_("Bad action character '%c'\n"), s[0]);
+      return;
+    }
+
+  while(loc<s.size())
+    {
+      while(loc<s.size() && isspace(s[loc]))
+	++loc;
+
+      if(loc<s.size())
+	{
+	  if(!parse_action_str(s, loc, action))
+	    {
+	      string pkgname;
+	      while(loc<s.size() && !isspace(s[loc]))
+		pkgname+=s[loc++];
+
+	      if(!cmdline_applyaction(pkgname, action,
+				      to_install, to_hold,
+				      to_remove, to_purge,
+				      verbose))
+		return;
+	    }
+	}
+    }
+}
