@@ -7,10 +7,8 @@
 
 #include "cmdline_common.h"
 
-class pkgProblemResolver;
-
 bool cmdline_applyaction(cmdline_pkgaction_type action,
-			 pkgProblemResolver &fixer, pkgCache::PkgIterator pkg,
+			 pkgCache::PkgIterator pkg,
 			 pkgset &to_install, pkgset &to_hold,
 			 pkgset &to_remove, pkgset &to_purge,
 			 int verbose,
@@ -19,9 +17,17 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
 
 bool cmdline_applyaction(string s,
 			 cmdline_pkgaction_type action,
-			 pkgProblemResolver &fixer,
 			 pkgset &to_install, pkgset &to_hold,
 			 pkgset &to_remove, pkgset &to_purge,
 			 int verbose);
 
+/** Parses a list of actions and executes them.  If there is a
+ *  syntax or other error, all the commands up to the point of
+ *  the error will be executed.
+ */
+void cmdline_parse_action(string s,
+			  pkgset &to_install, pkgset &to_hold,
+			  pkgset &to_remove, pkgset &to_purge,
+			  int verbose);
+
 #endif // CMDLINE_ACTION_H
