@@ -7,20 +7,23 @@
 #include "cmdline_action.h"
 #include "cmdline_common.h"
 #include "cmdline_prompt.h"
+#include "cmdline_resolver.h"
 #include "cmdline_show_broken.h"
 #include "cmdline_simulate.h"
 #include "cmdline_util.h"
 
-#include <generic/apt.h>
-#include <generic/config_signal.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/download_install_manager.h>
 
 #include <aptitude.h>
-#include <download.h>
 
 #include <apt-pkg/algorithms.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/progress.h>
 
+using namespace std;
+
 // TODO: add an option to obey sticky states even if it wasn't
 //      explicitly requested.
 //
@@ -30,7 +33,8 @@ int cmdline_do_action(int argc, char *argv[],
 		      const char *status_fname, bool simulate,
 		      bool assume_yes, bool download_only, bool fix_broken,
 		      bool showvers, bool showdeps, bool showsize,
-		      bool visual_preview, bool always_prompt, int verbose)
+		      bool visual_preview, bool always_prompt,
+		      bool queue_only, int verbose)
 {
   _error->DumpErrors();
 
@@ -53,6 +57,8 @@ int cmdline_do_action(int argc, char *argv[],
     default_action=cmdline_purge;
   else if(!strcasecmp(argv[0], "hold"))
     default_action=cmdline_hold;
+  else if(!strcasecmp(argv[0], "keep") || !strcasecmp(argv[0], "keep-all"))
+    default_action=cmdline_keep;
   else if(!strcasecmp(argv[0], "unhold"))
     default_action=cmdline_unhold;
   else if(!strcasecmp(argv[0], "markauto"))
@@ -69,7 +75,7 @@ int cmdline_do_action(int argc, char *argv[],
       return -1;
     }
 
-  OpTextProgress progress;
+  OpTextProgress progress(aptcfg->FindI("Quiet", 0));
 
   aptcfg->SetNoUser(PACKAGE "::Auto-Upgrade", "false");
 
@@ -99,7 +105,7 @@ int cmdline_do_action(int argc, char *argv[],
       return -1;
     }
 
-  pkgset to_install, to_hold, to_remove, to_purge;
+  pkgset to_upgrade, to_install, to_hold, to_remove, to_purge;
 
   if(dist_upgrade)
     {
@@ -110,82 +116,94 @@ int cmdline_do_action(int argc, char *argv[],
 
 	  if(!i.CurrentVer().end() &&
 	     state.Upgradable() && !(*apt_cache_file)->is_held(i))
-	    to_install.insert(i);
+	    to_upgrade.insert(i);
 
 	}
 
-      (*apt_cache_file)->mark_all_upgradable(true, NULL);
+      (*apt_cache_file)->mark_all_upgradable(true, false, NULL);
     }
-  else if(argc==1 && default_action==cmdline_install)
+  /*else if(argc==1 && default_action==cmdline_install)
     {
       // FIXME: Build to_install to avoid a big printout
       for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin(); !i.end(); ++i)
 	{
 
 	}
-    }
+	}*/
 
   // TODO: look for filenames and call dpkg directly if that's the case.
 
-  pkgProblemResolver fixer(*apt_cache_file);
-  // Used when doing only a local optimization (ie, we aren't trying to
-  // do a dist-upgrade)
-
   (*apt_cache_file)->begin_action_group();
-  // Mark packages 'n stuff.
-  for(int i=1; i<argc; ++i)
+  // If keep-all is the argument, we expect no patterns and keep all
+  // packages back.
+  if(!strcasecmp(argv[0], "keep-all"))
     {
-      cmdline_pkgaction_type action=default_action;
-      int tmp=strlen(argv[i])-1;
-
-      // HACK: disable interpreting of escapes if it's an existing
-      //      package name.
-      if((*apt_cache_file)->FindPkg(argv[i]).end())
-	switch(argv[i][tmp])
-	  {
-	  case '-':
-	    action=cmdline_remove;
-	    argv[i][tmp]=0;
-	    break;
-	  case '=':
-	    action=cmdline_hold;
-	    argv[i][tmp]=0;
-	    break;
-	  case '+':
-	    action=cmdline_install;
-	    argv[i][tmp]=0;
-	    break;
-	  case '_':
-	    action=cmdline_purge;
-	    argv[i][tmp]=0;
-	    break;
-	  }
-
-      cmdline_applyaction(argv[i], action, fixer,
-			  to_install, to_hold, to_remove, to_purge,
-			  verbose);
-    }
-  (*apt_cache_file)->end_action_group(NULL);
-
-  if(dist_upgrade ||
-     (fix_broken && argc==1 && default_action==cmdline_install))
-    {
-      if(!(*apt_cache_file)->try_fix_broken(NULL))
+      if(argc != 1)
 	{
-	  _error->DumpErrors();
+	  cerr << _("Unexpected pattern argument following \"keep-all\"") << endl;
 	  return -1;
 	}
+
+      for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin();
+	  !i.end(); ++i)
+	(*apt_cache_file)->mark_keep(i, true, false, NULL);
     }
   else
-    {
-      if(!(*apt_cache_file)->try_fix_broken(fixer, NULL))
-	_error->Error("Unable to resolve some dependencies!");
-    }
+    // If we didn't take the keep-all path, mark according to the
+    // action requested.
+    for(int i=1; i<argc; ++i)
+      {
+	cmdline_pkgaction_type action=default_action;
+	int tmp=strlen(argv[i])-1;
 
-  _error->DumpErrors();
+	// HACK: disable interpreting of escapes if it's an existing
+	//      package name.
+	if((*apt_cache_file)->FindPkg(argv[i]).end())
+	  switch(argv[i][tmp])
+	    {
+	    case '-':
+	      action=cmdline_remove;
+	      argv[i][tmp]=0;
+	      break;
+	    case '=':
+	      action=cmdline_hold;
+	      argv[i][tmp]=0;
+	      break;
+	    case '+':
+	      action=cmdline_install;
+	      argv[i][tmp]=0;
+	      break;
+	    case '_':
+	      action=cmdline_purge;
+	      argv[i][tmp]=0;
 
-  if(!show_broken())
-    return -1;
+	      break;
+	    case ':':
+	      action=cmdline_keep;
+	      argv[i][tmp]=0;
+	      break;
+	    case 'm':
+	    case 'M':
+	      if(tmp>0 && argv[i][tmp-1]=='&')
+		{
+		  argv[i][tmp-1]=0;
+		  if(argv[i][tmp]=='m')
+		    action=cmdline_unmarkauto;
+		  else
+		    action=cmdline_markauto;
+		}
+	      else if(tmp>0 && argv[i][tmp-1]=='+' && argv[i][tmp]=='M')
+		{
+		  argv[i][tmp-1]=0;
+		  action=cmdline_installauto;
+		}
+	    }
+
+	cmdline_applyaction(argv[i], action,
+			    to_install, to_hold, to_remove, to_purge,
+			    verbose);
+      }
+  (*apt_cache_file)->end_action_group(NULL);
 
   if(visual_preview)
     {
@@ -195,22 +213,34 @@ int cmdline_do_action(int argc, char *argv[],
   else if(simulate)
     return cmdline_simulate(dist_upgrade, to_install, to_hold, to_remove, to_purge,
 			    showvers, showdeps, showsize,
-			    always_prompt, verbose, assume_yes);
+			    always_prompt, verbose, assume_yes,
+			    !fix_broken);
+  else if(queue_only)
+    {
+      if(!(*apt_cache_file)->save_selection_list(progress))
+	return -1;
+      else
+	return 0;
+    }
   else
     {
       if(!cmdline_do_prompt(dist_upgrade,
 			    to_install, to_hold, to_remove, to_purge,
 			    showvers, showdeps, showsize,
-			    always_prompt, verbose, assume_yes))
+			    always_prompt, verbose, assume_yes,
+			    !fix_broken))
 	{
 	  printf(_("Abort.\n"));
 	  return 0;
 	}
 
-      int rval=do_install_run(&progress, true, download_only)?0:-1;
+      download_install_manager m(download_only);
+
+      int rval =
+	(cmdline_do_download(&m) == download_manager::success ? 0 : -1);
 
       if(_error->PendingError())
-	rval=-1;
+	rval = -1;
 
       _error->DumpErrors();
 
