@@ -8,11 +8,14 @@
 #include <desc_parse.h>
 
 #include "cmdline_common.h"
+#include "cmdline_util.h"
 
-#include <generic/apt.h>
-#include <generic/matchers.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/matchers.h>
 
 #include <vscreen/fragment.h>
+#include <vscreen/transcode.h>
 
 #include <apt-pkg/error.h>
 #include <apt-pkg/init.h>
@@ -26,17 +29,17 @@
 
 using namespace std;
 
-static ostream &operator<<(ostream &out, const fragment_contents &contents)
+ostream &operator<<(ostream &out, const fragment_contents &contents)
 {
   for(fragment_contents::const_iterator i=contents.begin();
       i!=contents.end(); ++i)
     {
-      string s;
-      // Drop the attributes for now.
-      for(chstring::const_iterator j=i->begin(); j!=i->end(); ++j)
-	s.push_back((*j)&A_CHARTEXT);
+      wstring s;
+      // Drop the attributes.
+      for(fragment_line::const_iterator j=i->begin(); j!=i->end(); ++j)
+	s.push_back((*j).ch);
 
-      out << s << endl;
+      out << transcode(s) << endl;
     }
 
   return out;
@@ -74,7 +77,7 @@ static fragment *dep_lst_frag(pkgCache::DepIterator dep,
 	  } while(1);
 
       if(!or_fragments.empty())
-	fragments.push_back(join_fragments(or_fragments, " | "));
+	fragments.push_back(join_fragments(or_fragments, L" | "));
     }
 
   if(fragments.size()==0)
@@ -83,7 +86,7 @@ static fragment *dep_lst_frag(pkgCache::DepIterator dep,
     return fragf("%s: %F",
 		 title.c_str(),
 		 indentbox(0, title.size()+2,
-			   flowbox(join_fragments(fragments, ", "))));
+			   flowbox(join_fragments(fragments, L", "))));
 }
 
 static fragment *prv_lst_frag(pkgCache::PrvIterator prv,
@@ -105,7 +108,7 @@ static fragment *prv_lst_frag(pkgCache::PrvIterator prv,
     return fragf("%s: %F",
 		 title.c_str(),
 		 indentbox(0, title.size()+2,
-			   flowbox(join_fragments(fragments, ", "))));
+			   flowbox(join_fragments(fragments, L", "))));
 }
 
 static fragment *archive_lst_frag(pkgCache::VerFileIterator vf,
@@ -114,7 +117,12 @@ static fragment *archive_lst_frag(pkgCache::VerFileIterator vf,
   vector<fragment *> fragments;
 
   for( ; !vf.end(); ++vf)
-    fragments.push_back(text_fragment(vf.File().Archive()));
+    {
+      if(vf.File().Archive() == 0)
+	fragments.push_back(text_fragment(_("<NULL>")));
+      else
+	fragments.push_back(text_fragment(vf.File().Archive()));
+    }
 
   if(fragments.size()==0)
     return fragf("");
@@ -122,7 +130,7 @@ static fragment *archive_lst_frag(pkgCache::VerFileIterator vf,
     return fragf("%s: %F",
 		 title.c_str(),
 		 indentbox(0, title.size()+2,
-			   flowbox(join_fragments(fragments, ", "))));
+			   flowbox(join_fragments(fragments, L", "))));
 }
 
 static const char *current_state_string(pkgCache::PkgIterator pkg)
@@ -238,7 +246,7 @@ static void show_package(pkgCache::PkgIterator pkg, int verbose)
 
   fragment *f=sequence_fragment(fragments);
 
-  cout << f->layout(screen_width);
+  cout << f->layout(screen_width, screen_width, style());
 
   delete f;
 }
@@ -296,7 +304,9 @@ static fragment *version_file_fragment(pkgCache::VerIterator ver,
       if(verbose<2) // Show all archives in a list.
 	fragments.push_back(archive_lst_frag(ver.FileList(), _("Archive")));
       else
-	fragments.push_back(fragf("%s: %s%n", _("Archive"), vf.File().Archive()));
+	{
+	  fragments.push_back(fragf("%s: %s%n", _("Archive"), vf.File().Archive()?vf.File().Archive():_("<NULL>")));
+	}
     }
 
   fragments.push_back(dep_lst_frag(ver.DependsList(),
@@ -319,7 +329,11 @@ static fragment *version_file_fragment(pkgCache::VerIterator ver,
 
   fragments.push_back(fragf("%s%s%n",
 			    _("Description: "), rec.ShortDesc().c_str()));
-  fragments.push_back(indentbox(1, 1, make_desc_fragment(rec.LongDesc())));
+  fragments.push_back(indentbox(1, 1, make_desc_fragment(transcode(rec.LongDesc()))));
+
+  fragment *tags = make_tags_fragment(pkg);
+  if(tags)
+    fragments.push_back(fragf("%n%F", tags));
 
   return sequence_fragment(fragments);
 }
@@ -330,7 +344,7 @@ static void show_version(pkgCache::VerIterator ver, int verbose)
     {
       fragment *f=version_file_fragment(ver, ver.FileList(), verbose);
 
-      cout << f->layout(screen_width);
+      cout << f->layout(screen_width, screen_width, style());
 
       delete f;
     }
@@ -340,7 +354,7 @@ static void show_version(pkgCache::VerIterator ver, int verbose)
 	{
 	  fragment *f=version_file_fragment(ver, vf, verbose);
 
-	  cout << f->layout(screen_width) << endl;
+	  cout << f->layout(screen_width, screen_width, style()) << endl;
 
 	  delete f;
 
@@ -351,47 +365,82 @@ static void show_version(pkgCache::VerIterator ver, int verbose)
     }
 }
 
-bool do_cmdline_show(string s, int verbose)
+static
+bool do_cmdline_show_target(const pkgCache::PkgIterator &pkg,
+			    cmdline_version_source source,
+			    const string &sourcestr,
+			    int verbose,
+			    bool has_explicit_source)
 {
-  bool is_pattern=(s.find('~')!=s.npos);
-  pkgCache::PkgIterator pkg;
-
-  if(!is_pattern)
-    pkg=(*apt_cache_file)->FindPkg(s);
-  if(!is_pattern && !pkg.end())
+  if(verbose == 0 || has_explicit_source)
     {
-      if(!pkg.VersionList().end())
-	for(pkgCache::VerIterator V=pkg.VersionList(); !V.end(); ++V)
-	  {
-	    show_version(V, verbose);
-	    if(verbose==0)
-	      break;
-	  }
+      pkgCache::VerIterator ver = cmdline_find_ver(pkg, source, sourcestr);
+
+      if(ver.end())
+	ver = pkg.VersionList();
+
+      if(!ver.end())
+	show_version(ver, verbose);
       else
 	show_package(pkg, verbose);
     }
+  else if(!pkg.VersionList().end())
+    for(pkgCache::VerIterator ver=pkg.VersionList(); !ver.end(); ++ver)
+      show_version(ver, verbose);
+  else
+    show_package(pkg, verbose);
+
+  return true;
+}
+
+bool do_cmdline_show(string s, int verbose)
+{
+  cmdline_version_source source;
+  string name, sourcestr;
+  string default_release = aptcfg->Find("APT::Default-Release");
+  bool has_explicit_source = false;
+
+  if(!cmdline_parse_source(s, source, name, sourcestr))
+    return false;
+
+  has_explicit_source = (source != cmdline_version_cand);
+
+  if(source == cmdline_version_cand && !default_release.empty())
+    {
+      source    = cmdline_version_archive;
+      sourcestr = default_release;
+    }
+
+  bool is_pattern=(name.find('~')!=name.npos);
+  pkgCache::PkgIterator pkg;
+
+  if(!is_pattern)
+    {
+      pkg=(*apt_cache_file)->FindPkg(name);
+
+      if(pkg.end())
+	{
+	  _error->Error(_("Unable to locate package %s"), s.c_str());
+	  return false;
+	}
+    }
+
+  if(!is_pattern && !pkg.end())
+    return do_cmdline_show_target(pkg, source, sourcestr, verbose, has_explicit_source);
   else if(is_pattern)
     {
-      pkg_matcher *m=parse_pattern(s);
+      pkg_matcher *m=parse_pattern(name);
 
       if(!m)
 	{
-	  _error->Error("Unable to parse pattern %s", s.c_str());
-	  _error->DumpErrors();
+	  _error->Error(_("Unable to parse pattern %s"), name.c_str());
 	  return false;
 	}
 
       for(pkgCache::PkgIterator P=(*apt_cache_file)->PkgBegin();
 	  !P.end(); ++P)
-	{
-	  for(pkgCache::VerIterator V=P.VersionList(); !V.end(); ++V)
-	    if(m->matches(P, V))
-	      {
-		show_version(V, verbose);
-		if(verbose==0)
-		  break;
-	      }
-	}
+	if(m->matches(P))
+	  return do_cmdline_show_target(P, source, sourcestr, verbose, has_explicit_source);
 
       delete m;
     }
@@ -404,8 +453,6 @@ bool do_cmdline_show(string s, int verbose)
 
 int cmdline_show(int argc, char *argv[], int verbose)
 {
-  update_screen_width();
-
   _error->DumpErrors();
 
   OpProgress progress;
@@ -419,7 +466,16 @@ int cmdline_show(int argc, char *argv[], int verbose)
 
   for(int i=1; i<argc; ++i)
     if(!do_cmdline_show(argv[i], verbose))
+      {
+	_error->DumpErrors();
+	return -1;
+      }
+
+  if(_error->PendingError())
+    {
+      _error->DumpErrors();
       return -1;
+    }
 
   return 0;
 }
