@@ -9,7 +9,7 @@
 
 #include <aptitude.h>
 
-#include <generic/apt.h>
+#include <generic/apt/apt.h>
 
 #include <apt-pkg/algorithms.h>
 #include <apt-pkg/error.h>
@@ -20,12 +20,13 @@ int cmdline_simulate(bool as_upgrade,
 		     pkgset &to_install, pkgset &to_hold, pkgset &to_remove,
 		     pkgset &to_purge,
 		     bool showvers, bool showdeps, bool showsize,
-		     bool always_prompt, int verbose, bool assume_yes)
+		     bool always_prompt, int verbose,
+		     bool assume_yes, bool force_no_change)
 {
   if(!cmdline_do_prompt(as_upgrade,
 			to_install, to_hold, to_remove, to_purge,
 			showvers, showdeps, showsize, always_prompt, verbose,
-			assume_yes))
+			assume_yes, force_no_change))
     {
       printf(_("Abort.\n"));
       return 0;
