@@ -5,23 +5,43 @@
 #include "cmdline_util.h"
 
 #include "cmdline_common.h"
+#include "cmdline_progress.h"
 
 #include <aptitude.h>
 #include <ui.h>
+#include <vs_progress.h>
 
-#include <generic/apt.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/download_manager.h>
+#include <generic/apt/download_signal_log.h>
 
 #include <vscreen/vscreen.h>
 
+#include <sigc++/functors/ptr_fun.h>
+
 void ui_preview()
 {
   ui_init();
-  file_quit.connect(SigC::slot(vscreen_exitmain));
+  file_quit.connect(sigc::ptr_fun(vscreen_exitmain));
   do_package_run_or_show_preview();
   ui_main();
   exit(0);
 }
 
+void ui_solution_screen()
+{
+  ui_init();
+  file_quit.connect(sigc::ptr_fun(vscreen_exitmain));
+
+  vs_progress_ref p = gen_progress_bar();
+  do_new_package_view(*p.unsafe_get_ref());
+
+  do_examine_solution();
+  ui_main();
+  exit(0);
+}
+
 void cmdline_show_stringlist(strvector &items)
 {
   int loc=2;
@@ -139,3 +159,23 @@ bool cmdline_parse_source(const string &input,
 
   return true;
 }
+
+download_manager::result cmdline_do_download(download_manager *m)
+{
+  OpTextProgress progress(aptcfg->FindI("Quiet", 0));
+  std::auto_ptr<download_signal_log> log(gen_cmdline_download_progress());
+
+  if(!m->prepare(progress, *log.get(), log.get()))
+    return download_manager::failure;
+
+  download_manager::result finish_res;
+
+  do
+    {
+      pkgAcquire::RunResult download_res = m->do_download();
+      finish_res = m->finish(download_res, progress);
+    }
+  while(finish_res == download_manager::do_again);
+
+  return finish_res;
+}
