@@ -1,6 +1,6 @@
 // dep_item.cc
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,11 +24,12 @@
 #include "aptitude.h"
 
 #include "dep_item.h"
+#include "pkg_sortpolicy.h"
 #include "pkg_subtree.h"
 #include "pkg_ver_item.h"
 
-#include "generic/apt.h"
-#include "generic/config_signal.h"
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
 
 #include <apt-pkg/version.h>
 
@@ -48,9 +49,10 @@ class pkg_depitem:public pkg_subtree
 public:
   pkg_depitem(pkgCache::DepIterator &_first, pkg_signal *_sig);
 
-  void paint(vs_tree *win, int y, bool hierarchical);
+  void paint(vs_tree *win, int y, bool hierarchical,
+	     const style &st);
 
-  int get_normal_attr();
+  style get_normal_style();
 
   void select(undo_group *undo);
   // A special case -- installs the 'best' target package (using
@@ -58,7 +60,7 @@ public:
 };
 
 pkg_depitem::pkg_depitem(pkgCache::DepIterator &first, pkg_signal *sig):
-  pkg_subtree(""),firstdep(first),available(false)
+  pkg_subtree(L""),firstdep(first),available(false)
 {
   pkgCache::DepIterator start,end;
   first.GlobOr(start,end);
@@ -103,7 +105,7 @@ pkg_depitem::pkg_depitem(pkgCache::DepIterator &first, pkg_signal *sig):
       // Bleach.  Anyone who can offer a cleaner way to do this gets a virtual cookie :)
       start++;
     } while(1);
-  pkg_subtree::set_label(currlabel.c_str());
+  pkg_subtree::set_label(transcode(currlabel, "ASCII"));
 }
 
 bool pkg_depitem::is_broken()
@@ -126,7 +128,8 @@ bool pkg_depitem::is_broken()
 	  !((*apt_cache_file)[dep]&pkgDepCache::DepGInstall));
 }
 
-void pkg_depitem::paint(vs_tree *win, int y, bool hierarchical)
+void pkg_depitem::paint(vs_tree *win, int y, bool hierarchical,
+			const style &st)
 {
   if(is_broken())
     {
@@ -134,21 +137,22 @@ void pkg_depitem::paint(vs_tree *win, int y, bool hierarchical)
       // the user that.  (specifically, if there are no available packages
       // fulfilling the dependency)
 
-      string broken_str=available?_("UNSATISFIED"):_("UNAVAILABLE");
+      wstring broken_str=transcode(available?_("UNSATISFIED"):_("UNAVAILABLE"));
 
       vs_subtree<pkg_tree_node>::paint(win, y, hierarchical,
-				       get_name()+" ("+broken_str+")");
+				       get_name()+L" ("+broken_str+L")");
     }
   else
-    pkg_subtree::paint(win, y, hierarchical);
+    pkg_subtree::paint(win, y, hierarchical,
+		       st);
 }
 
-int pkg_depitem::get_normal_attr()
+style pkg_depitem::get_normal_style()
 {
   if(is_broken())
-    return get_color("DepBroken");
+    return get_style("DepBroken");
   else
-    return pkg_subtree::get_normal_attr();
+    return pkg_subtree::get_normal_style();
 }
 
 void pkg_depitem::select(undo_group *undo)
@@ -257,7 +261,8 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
       tree_map::iterator found=subtrees.find(D.DepType());
       if(found==subtrees.end())
 	{
-	  subtree=new pkg_subtree(D.DepType(), true);
+	  subtree=new pkg_subtree(transcode(D.DepType()),
+				  true);
 	  subtrees[D.DepType()]=subtree;
 	  tree->add_child(subtree);
 	}
@@ -286,7 +291,8 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
 		tree_map::iterator found=subtrees.find(D.DepType());
 		if(found==subtrees.end())
 		  {
-		    subtree=new pkg_subtree(D.DepType(), true);
+		    subtree=new pkg_subtree(transcode(D.DepType()),
+					    true);
 		    subtrees[D.DepType()]=subtree;
 		    tree->add_child(subtree);
 		  }
@@ -297,6 +303,13 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
 	      }
 	  }
       }
+
+  pkg_sortpolicy *policy = pkg_sortpolicy_name(pkg_sortpolicy_ver(NULL, false), false);
+  pkg_sortpolicy_wrapper sorter(policy);
+  for(tree_map::const_iterator i = subtrees.begin();
+      i != subtrees.end(); ++i)
+    i->second->sort(sorter);
+  delete policy;
 }
 
 template void setup_package_deps<pkg_subtree>(const pkgCache::PkgIterator &pkg,
