@@ -1,6 +1,6 @@
 // download_item.cc
 //
-//  Copyright 1999, 2004, 2005 Daniel Burrows
+//  Copyright 1999, 2004-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,32 +22,35 @@
 #include "download_item.h"
 
 #include <apt-pkg/acquire-worker.h>
-#include <apt-pkg/init.h> // For the APT version number.
 #include <apt-pkg/strutl.h>
 
-#include "vscreen/vs_tree.h"
+#include <vscreen/transcode.h>
+#include <vscreen/vs_tree.h>
 
-int download_item::get_normal_attr()
+style download_item::get_normal_style()
 {
   switch(item.Owner->Status)
     {
     case pkgAcquire::Item::StatIdle:
-      return A_DIM|vs_treeitem::get_normal_attr();
+      return vs_treeitem::get_normal_style()+style_attrs_on(A_DIM);
     case pkgAcquire::Item::StatFetching:
-      return vs_treeitem::get_normal_attr();
+      return vs_treeitem::get_normal_style();
     case pkgAcquire::Item::StatDone:
       if(!hit)
-	return get_color("Progress");
+	return vs_treeitem::get_normal_style()+get_style("Progress");
       else
-	return get_color("DownloadHit");
+	return vs_treeitem::get_normal_style()+get_style("DownloadHit");
     case pkgAcquire::Item::StatError:
-      return get_color("Error");
+      return vs_treeitem::get_normal_style()+get_style("Error");
+    case pkgAcquire::Item::StatAuthError:
+      return vs_treeitem::get_normal_style()+get_style("Error");
     default:
       assert(0);
     }
 }
 
-void download_item::paint(vs_tree *win, int y, bool hierarchical)
+void download_item::paint(vs_tree *win, int y, bool hierarchical,
+			  const style &st)
   // A little confusing -- basically, there are two branches: either we display
   // a progress bar, or we don't.  If we don't, I can just display it as usual
   // (note, though, that I don't yet indent it according to the depth..); if we
@@ -59,6 +62,10 @@ void download_item::paint(vs_tree *win, int y, bool hierarchical)
 
   win->getmaxyx(height,width);
 
+  const style progress_style=st+get_style("DownloadProgress");
+  const style normal_style=st+get_normal_style();
+  int barsize=width;
+
   switch(item.Owner->Status)
     {
     case pkgAcquire::Item::StatIdle:
@@ -74,26 +81,18 @@ void download_item::paint(vs_tree *win, int y, bool hierarchical)
 	  if(worker->TotalSize>0)
 	    {
 	      char intbuf[50]; // Waay more than enough.
-	      int barsize=(width*worker->CurrentSize)/worker->TotalSize;
-	      win->attrset(get_color("DownloadProgress"));
+	      barsize=(width*worker->CurrentSize)/worker->TotalSize;
+	      win->apply_style(progress_style);
 
 	      if(barsize>width)
 		barsize=width;
 
-	      win->attrset(get_normal_attr());
+	      win->apply_style(get_normal_style());
 	      sprintf(intbuf,
 		      "%sB/%sB",
 		      SizeToStr(worker->CurrentSize).c_str(),
 		      SizeToStr(worker->TotalSize).c_str());
 	      output+=string(" [ ")+intbuf+" ]";
-	      win->show_string_as_progbar(0,
-					  y,
-					  output,
-					  get_color("DownloadProgress"),
-					  get_normal_attr(),
-					  barsize,
-					  width);
-	      return;
 	    }
 	  else
 	    output+=_(" [Working]");
@@ -105,19 +104,12 @@ void download_item::paint(vs_tree *win, int y, bool hierarchical)
     case pkgAcquire::Item::StatError:
       output+=item.Owner->ErrorText;
       break;
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
     case pkgAcquire::Item::StatAuthError:
       output+=item.Owner->ErrorText;
       break;
-#endif
     }
 
-  win->mvaddnstr(y, 0, output.c_str(), width);
-
-  int newy,newx;
-  win->getyx(newy, newx);
-  if(newy!=y)
-    return;
-  for( ; newx<width; newx++)
-    win->addch(' ');
+  win->show_string_as_progbar(0, y, transcode(output),
+			      progress_style, normal_style,
+			      barsize, width);
 }
