@@ -1,6 +1,6 @@
 // download_screen.h  (this is -*-c++-*- )
 //
-//  Copyright 1999 Daniel Burrows
+//  Copyright 1999-2001, 2003-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #define DOWNLOAD_SCREEN_H
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include <config.h>
 #endif
 
 #include <apt-pkg/acquire.h>
@@ -60,10 +60,10 @@ class download_tree:public vs_subtree_generic
 public:
   download_tree():vs_subtree_generic(true) {}
 
-  void paint(vs_tree *win, int y, bool hierarchical)
-  {vs_subtree_generic::paint(win, y, hierarchical, "ERROR: SHOULD NOT APPEAR");}
-  const char * tag() const {return "download tree";}
-  const char * label() const {return "download tree";}
+  void paint(vs_tree *win, int y, bool hierarchical, const style &style)
+  {vs_subtree_generic::paint(win, y, hierarchical, L"ERROR: SHOULD NOT APPEAR");}
+  const wchar_t * tag() {return L"download tree";}
+  const wchar_t * label() {return L"download tree";}
 };
 
 class download_screen:public vs_tree, public pkgAcquireStatus
@@ -95,7 +95,7 @@ class download_screen:public vs_tree, public pkgAcquireStatus
   }
 
 protected:
-  bool handle_char(chtype ch);
+  bool handle_key(const key &k);
 
 public:
   download_screen():prev(NULL),finished(false),cancelled(false) {contents=new download_tree; set_root(contents);}
