@@ -1,46 +1,70 @@
 // edit_pkg_hier.cc
+//
+//   Copyright (C) 2000-2001, 2004-2005 Daniel Burrows
+//
+//   This program is free software; you can redistribute it and/or
+//   modify it under the terms of the GNU General Public License as
+//   published by the Free Software Foundation; either version 2 of
+//   the License, or (at your option) any later version.
+//
+//   This program is distributed in the hope that it will be useful,
+//   but WITHOUT ANY WARRANTY; without even the implied warranty of
+//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//   General Public License for more details.
+//
+//   You should have received a copy of the GNU General Public License
+//   along with this program; see the file COPYING.  If not, write to
+//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+//   Boston, MA 02111-1307, USA.
+
 
 #include "edit_pkg_hier.h"
 
 #include "aptitude.h"
+#include "ui.h"
 
 #include <apt-pkg/error.h>
 
-#include <sigc++/bind.h>
-#include <sigc++/object.h>
-#include <sigc++/object_slot.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/functors/mem_fun.h>
+#include <sigc++/trackable.h>
 
-#include <generic/apt.h>
+#include <generic/apt/apt.h>
+
+#include <generic/util/util.h>
 
 #include <vscreen/config/keybindings.h>
 #include <vscreen/vs_subtree.h>
+#include <vscreen/transcode.h>
 
 using namespace std;
 using namespace __gnu_cxx;
 
 // Stores a group (name) and a Y/N state
-class vs_hier_editor::vs_hier_item:public SigC::Object, public vs_treeitem
+class vs_hier_editor::vs_hier_item:public sigc::trackable, public vs_treeitem
 {
   bool selected;
   pkg_hier::group *group;
   pkg_hier::item *item;
+  // whatever
+  wstring group_name;
 public:
   vs_hier_item(pkg_hier::group *_group, pkg_hier::item *_item)
-    :vs_treeitem(true), group(_group)
+    :vs_treeitem(true), group(_group), group_name(transcode(group->name, "ASCII"))
   {
     set_item(_item);
   }
 
-  bool dispatch_char(chtype ch, vs_tree *owner)
+  bool dispatch_key(const key &k, vs_tree *owner)
   {
-    if(global_bindings.key_matches(ch, "PushButton") ||
-       global_bindings.key_matches(ch, "Confirm"))
+    if(global_bindings.key_matches(k, "PushButton") ||
+       global_bindings.key_matches(k, "Confirm"))
       {
 	selected=!selected;
 	vscreen_update();
       }
     else
-      return vs_treeitem::dispatch_char(ch, owner);
+      return vs_treeitem::dispatch_key(k, owner);
 
     return true;
   }
@@ -57,7 +81,7 @@ public:
       vs_treeitem::dispatch_mouse(id, x, bstate, owner);
   }
 
-  void paint(vs_tree *win, int y, bool hierarchical)
+  void paint(vs_tree *win, int y, bool hierarchical, const style &st)
   {
     string::size_type width=win->get_width();
     string todisp=" ";
@@ -75,17 +99,17 @@ public:
     while(todisp.size()<width)
       todisp+=" ";
 
-    win->mvaddnstr(y, 0, todisp.c_str(), width);
+    win->mvaddnstr(y, 0, transcode(todisp, "ASCII"), width);
   }
 
-  const char *tag() const
+  const wchar_t *tag()
   {
-    return group->name.c_str();
+    return group_name.c_str();
   }
 
-  const char *label() const
+  const wchar_t *label()
   {
-    return group->name.c_str();
+    return group_name.c_str();
   }
 
   void commit()
@@ -114,27 +138,29 @@ public:
 // FIXME: I shouldn't have to do this.
 class silly_subtree:public vs_subtree_generic
 {
-  string txt;
+  wstring txt;
 public:
-  silly_subtree(bool expanded, string _txt)
+  silly_subtree(bool expanded, const wstring &_txt)
     :vs_subtree_generic(expanded), txt(_txt) {}
 
-  void paint(vs_tree *win, int y, bool hierarchical)
+  void paint(vs_tree *win, int y, bool hierarchical, const style &st)
   {
     vs_subtree_generic::paint(win, y, hierarchical, txt);
   }
 
-  const char *tag() const {return txt.c_str();}
-  const char *label() const {return txt.c_str();}
+  const wchar_t *tag() {return txt.c_str();}
+  const wchar_t *label() {return txt.c_str();}
 };
 
 vs_hier_editor::vs_hier_editor():item(NULL)
 {
-  hier_reloaded.connect(slot(*this, &vs_hier_editor::handle_reload));
+  hier_reloaded.connect(sigc::mem_fun(*this, &vs_hier_editor::handle_reload));
 }
 
 void vs_hier_editor::handle_reload()
 {
+  vs_widget_ref tmpref(this);
+
   set_root(NULL);
   items.clear();
 
@@ -152,12 +178,12 @@ bool vs_hier_editor::get_cursorvisible()
     return true;
 }
 
-void vs_hier_editor::paint()
+void vs_hier_editor::paint(const style &st)
 {
   if(!item)
     mvaddnstr(0, 0, _("No hierarchy information to edit"), get_width());
   else
-    vs_tree::paint();
+    vs_tree::paint(st);
 }
 
 // Creates a new list of edit-widgets if pkg isn't an end iterator.
@@ -165,6 +191,8 @@ void vs_hier_editor::paint()
 // (yes, it would be nicer in some ways to not recreate the tree continually)
 void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
 {
+  vs_widget_ref tmpref(this);
+
   shown_conn.disconnect();
   if(get_visible())
     {
@@ -187,7 +215,7 @@ void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
 
 	  if(items.empty())
 	    {
-	      silly_subtree *newroot=new silly_subtree(true, "All groups");
+	      silly_subtree *newroot=new silly_subtree(true, L"All groups");
 
 	      // Add all available groups to the list.
 	      for(pkg_hier::groupmap::iterator i=user_pkg_hier->groups.begin();
@@ -195,7 +223,7 @@ void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
 		  ++i)
 		{
 		  vs_hier_item *tmp=new vs_hier_item(&i->second, item);
-		  commit_changes.connect(SigC::slot(*tmp, &vs_hier_item::commit));
+		  commit_changes.connect(sigc::mem_fun(*tmp, &vs_hier_item::commit));
 
 		  newroot->add_child(tmp);
 		  items.push_back(tmp);
@@ -217,9 +245,9 @@ void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
     {
       string name=pkg.end()?"":pkg.Name();
 
-      shown_conn=shown_sig.connect(bind(slot(*this,
-					     (void (vs_hier_editor::*) (string)) &vs_hier_editor::set_package),
-					name));
+      shown_conn=shown_sig.connect(sigc::bind(sigc::mem_fun(*this,
+							    (void (vs_hier_editor::*) (string)) &vs_hier_editor::set_package),
+					      name));
     }
 }
 
@@ -291,16 +319,25 @@ void vs_hier_editor::save_hier(string file)
   fclose(f);
 }
 
-bool vs_hier_editor::handle_char(chtype ch)
+bool vs_hier_editor::handle_key(const key &k)
 {
-  if(global_bindings.key_matches(ch, "SaveHier"))
+  if(global_bindings.key_matches(k, "SaveHier"))
     {
-      const char *cfgloc=getenv("HOME");
+      string homedir = get_homedir();
+      string cfgfile;
 
-      string cfgfile=string(cfgloc)+"/.aptitude/function_pkgs";
+      if(homedir.empty())
+	{
+	  show_message(_("Unable to look up your home directory, saving to /tmp/function_pkgs!"),
+		       NULL,
+		       get_style("Error"));
+	  cfgfile = "/tmp/function_pkgs";
+	}
+      else
+	cfgfile = homedir + "/.aptitude/function_pkgs";
       save_hier(cfgfile);
     }
-  else if(global_bindings.key_matches(ch, "Quit"))
+  else if(global_bindings.key_matches(k, "Quit"))
     {
       if(item)
 	{
@@ -313,7 +350,7 @@ bool vs_hier_editor::handle_char(chtype ch)
 
       hide();
     }
-  else if(global_bindings.key_matches(ch, "Commit"))
+  else if(global_bindings.key_matches(k, "Commit"))
     {
       if(item)
 	{
@@ -326,10 +363,10 @@ bool vs_hier_editor::handle_char(chtype ch)
 
       commit_changes();
     }
-  else if(global_bindings.key_matches(ch, "Abort"))
+  else if(global_bindings.key_matches(k, "Abort"))
     hide();
   else
-    return vs_tree::handle_char(ch);
+    return vs_tree::handle_key(k);
 
   return true;
 }
