@@ -1,6 +1,6 @@
-// testscr.cc
+// main.cc  (ne testscr.cc)
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,31 +24,41 @@
 #endif
 
 #include <getopt.h>
+#include <signal.h>
 
 #include "aptitude.h"
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "generic/apt.h"
-#include "generic/config_signal.h"
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+
+#include <generic/problemresolver/exceptions.h>
 
 #include <vscreen/config/keybindings.h>
+#include <vscreen/transcode.h>
 #include <vscreen/vscreen.h>
 #include <vscreen/vs_util.h>
 
 #include <cmdline/cmdline_changelog.h>
 #include <cmdline/cmdline_clean.h>
+#include <cmdline/cmdline_common.h>
 #include <cmdline/cmdline_do_action.h>
 #include <cmdline/cmdline_download.h>
+#include <cmdline/cmdline_dump_resolver.h>
 #include <cmdline/cmdline_forget_new.h>
 #include <cmdline/cmdline_moo.h>
+#include <cmdline/cmdline_prompt.h>
 #include <cmdline/cmdline_search.h>
 #include <cmdline/cmdline_show.h>
 #include <cmdline/cmdline_update.h>
 #include <cmdline/cmdline_upgrade.h>
 
-#include <sigc++/slot.h>
+#include <sigc++/functors/ptr_fun.h>
 
+#include <apt-pkg/error.h>
 #include <apt-pkg/init.h>
 
 #include "ui.h"
@@ -58,11 +68,32 @@
 #include "pkg_grouppolicy.h"
 #include "pkg_view.h"
 
-static void do_vscreen_update(const pkgCache::PkgIterator &)
+#if 0
+// These are commented out so as to not punish users unduly for coding
+// errors.  The transcoder now substitutes conspicuous '?' characters
+// into its output, which should be enough of a clue.
+
+
+/** Handles a coding error using the apt error mechanism. */
+std::wstring handle_mbtow_error(int error,
+				const std::wstring &partial,
+				const std::string &input)
 {
-  vscreen_update();
+  _error->Errno("iconv", _("Can't decode multibyte string after \"%ls\""),
+		partial.c_str());
+  return partial;
 }
 
+std::string handle_wtomb_error(int error,
+			       const std::string &partial,
+			       const std::wstring &input)
+{
+  _error->Errno("iconv", _("Can't decode wide-character string after \"%s\""),
+		partial.c_str());
+  return partial;
+}
+#endif
+
 static void show_version()
 {
   printf(_("%s %s compiled at %s %s\n"),
@@ -108,6 +139,7 @@ static void usage()
   printf(_(" autoclean    - Erase old downloaded package files\n"));
   printf(_(" changelog    - View a package's changelog\n"));
   printf(_(" download     - Download the .deb file for a package\n"));
+  printf(_(" reinstall    - Download and (possibly) reinstall a currently installed package\n"));
   printf("\n");
   printf(_("  Options:\n"));
   printf(_(" -h             This help text\n"));
@@ -124,6 +156,7 @@ static void usage()
   printf(_(" -Z		    Show the change in installed size of each package.\n"));
   printf(_(" -v             Display extra information. (may be supplied multiple times)\n"));
   printf(_(" -t [release]   Set the release from which packages should be installed\n"));
+  printf(_(" -q             In command-line mode, suppress the incremental progress indicators.\n"));
   printf(_(" -o key=val     Directly set the configuration option named 'key'\n"));
   printf(_(" --with(out)-recommends	Specify whether or not to treat recommends as\n                strong dependencies\n"));
   printf(_(" -S fname: Read the aptitude extended status info from fname.\n"));
@@ -134,13 +167,14 @@ static void usage()
 }
 
 // This handles options with no single-character equivalent
-enum {OPTION_VERSION=1, OPTION_VISUAL_PREVIEW};
+enum {OPTION_VERSION=1, OPTION_VISUAL_PREVIEW, OPTION_QUEUE_ONLY};
 int getopt_result;
 
 option opts[]={
   {"help", 0, NULL, 'h'},
   {"version", 0, &getopt_result, OPTION_VERSION},
   {"display-format", 1, NULL, 'F'},
+  {"quiet", 2, NULL, 'q'},
   {"width", 1, NULL, 'w'},
   {"simulate", 0, NULL, 's'},
   {"with-recommends", 0, NULL, 'r'},
@@ -154,6 +188,7 @@ option opts[]={
   {"sort", 1, NULL, 'O'},
   {"target-release", 1, NULL, 't'},
   {"visual-preview", 0, &getopt_result, OPTION_VISUAL_PREVIEW},
+  {"schedule-only", 0, &getopt_result, OPTION_QUEUE_ONLY},
   {0,0,0,0}
 };
 
@@ -163,6 +198,11 @@ int main(int argc, char *argv[])
 {
   srandom(time(0));
 
+  // See earlier note
+  //
+  //transcode_mbtow_err=handle_mbtow_error;
+  //transcode_wtomb_err=handle_wtomb_error;
+
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
@@ -174,10 +214,11 @@ int main(int argc, char *argv[])
   string display_format=aptcfg->Find(PACKAGE "::CmdLine::Package-Display-Format", "%c%a%M %p# - %d#");
   string sort_policy="name";
   string width=aptcfg->Find(PACKAGE "::CmdLine::Package-Display-Width", "");
-  bool simulate=aptcfg->FindB(PACKAGE "::CmdLine::Simulate", false);
+  bool simulate = aptcfg->FindB(PACKAGE "::CmdLine::Simulate", false) ||
+    aptcfg->FindB(PACKAGE "::Simulate", false);
   bool download_only=aptcfg->FindB(PACKAGE "::CmdLine::Download-Only", false);;
 
-  bool update_only=false, install_only=false;
+  bool update_only=false, install_only=false, queue_only=false;
   bool assume_yes=aptcfg->FindB(PACKAGE "::CmdLine::Assume-Yes", false);
   bool fix_broken=aptcfg->FindB(PACKAGE "::CmdLine::Fix-Broken", false);
   bool showvers=aptcfg->FindB(PACKAGE "::CmdLine::Show-Versions", false);
@@ -186,6 +227,8 @@ int main(int argc, char *argv[])
   bool visual_preview=aptcfg->FindB(PACKAGE "::CmdLine::Visual-Preview", false);
   bool always_prompt=aptcfg->FindB(PACKAGE "::CmdLine::Always-Prompt", false);
   int verbose=aptcfg->FindI(PACKAGE "::CmdLine::Verbose", 0);
+  bool seen_quiet = false;
+  int quiet = 0;
 
   int curopt;
   // The last option seen
@@ -194,11 +237,19 @@ int main(int argc, char *argv[])
   //      rocker.
   argv0=argv[0];
 
-  // This will be used to hold current locale.
-  char *locale;
+  // Backwards bug-compatibility; old versions used a misleading name
+  // for this option.
+  if(aptcfg->Find(PACKAGE "::Keep-Unused-Pattern", "") == "")
+    {
+      aptcfg->Set(PACKAGE "::Keep-Unused-Pattern",
+		  aptcfg->Find(PACKAGE "::Delete-Unused-Pattern", ""));
+      aptcfg->Set(PACKAGE "::Delete-Unused-Pattern", "");
+    }
+  else
+    aptcfg->Set(PACKAGE "::Delete-Unused-Pattern", "");
 
   // Read the arguments:
-  while((curopt=getopt_long(argc, argv, "DVZvhS:uiF:w:sO:fdyPt:Rro:", opts, NULL))!=-1)
+  while((curopt=getopt_long(argc, argv, "DVZvhS:uiF:w:sO:fdyPt:q::Rro:", opts, NULL))!=-1)
     {
       switch(curopt)
 	{
@@ -235,6 +286,32 @@ int main(int argc, char *argv[])
 	case 'y':
 	  assume_yes=true;
 	  break;
+	case 'q':
+	  if(optarg == 0)
+	    ++quiet;
+	  else
+	    {
+	      if(*optarg == '=')
+		++optarg;
+
+	      if(*optarg == 0)
+		{
+		  fprintf(stderr, _("Expected a number after -q=\n"));
+		  return -1;
+		}
+
+	      char *tmp;
+	      quiet = strtol(optarg, &tmp, 0);
+
+	      if(*tmp != '\0')
+		{
+		  fprintf(stderr, _("Expected a number after -q=, got %s\n"),
+			  optarg);
+		  return -1;
+		}
+	    }
+	  seen_quiet = true;
+	  break;
 	case 'r':
 	  aptcfg->SetNoUser(PACKAGE "::Recommends-Important", "true");
 	  break;
@@ -286,14 +363,10 @@ int main(int argc, char *argv[])
 	      show_version();
 	      exit(0);
 	    case OPTION_VISUAL_PREVIEW:
-	      visual_preview=true;
-	      locale = setlocale(LC_ALL, NULL);    
-	      
-              if (strstr(locale, "UTF-8")) {
-	      	      setlocale(LC_ALL, "C");
-		      locale = setlocale(LC_ALL, NULL);
-              }
-	      
+	      visual_preview=true;	      
+	      break;
+	    case OPTION_QUEUE_ONLY:
+	      queue_only=true;
 	      break;
 	    default:
 	      fprintf(stderr, "%s",
@@ -311,6 +384,14 @@ int main(int argc, char *argv[])
 	}
     }
 
+  if(quiet == 0 && !isatty(1))
+    aptcfg->SetNoUser("quiet", 1);
+  else if(seen_quiet)
+    aptcfg->SetNoUser("quiet", quiet);
+
+  if(simulate)
+    aptcfg->SetNoUser(PACKAGE "::Simulate", true);
+
   // Sanity-check
   if(update_only && install_only)
     {
@@ -331,112 +412,155 @@ int main(int argc, char *argv[])
   // Possibly run off and do other commands.
   if(optind!=argc)
     {
-      if(update_only || install_only)
+      try
 	{
-	  fprintf(stderr, "%s",
-		  _("-u and -i may not be specified with a command"));
-	  usage();
-	  exit(1);
-	}
+	  // Connect up the "please consume errors" routine for the
+	  // command-line.
+	  consume_errors.connect(sigc::mem_fun(_error, &GlobalError::DumpErrors));
 
-      if(!strcasecmp(argv[optind], "update"))
-	return cmdline_update(argc-optind, argv+optind);
-      else if(!strcasecmp(argv[optind], "clean"))
-	return cmdline_clean(argc-optind, argv+optind, simulate);
-      else if(!strcasecmp(argv[optind], "autoclean"))
-	return cmdline_autoclean(argc-optind, argv+optind, simulate);
-      else if(!strcasecmp(argv[optind], "forget-new"))
-	return cmdline_forget_new(argc-optind, argv+optind,
-				  status_fname, simulate);
-      else if(!strcasecmp(argv[optind], "search"))
-	return cmdline_search(argc-optind, argv+optind,
-			      status_fname,
-			      display_format, width,
-			      sort_policy);
-      else if( (!strcasecmp(argv[optind], "install")) ||
-	       (!strcasecmp(argv[optind], "reinstall")) ||
-	       (!strcasecmp(argv[optind], "dist-upgrade")) ||
-	       (!strcasecmp(argv[optind], "remove")) ||
-	       (!strcasecmp(argv[optind], "purge")) ||
-	       (!strcasecmp(argv[optind], "hold")) ||
-	       (!strcasecmp(argv[optind], "unhold")) ||
-	       (!strcasecmp(argv[optind], "markauto")) ||
-	       (!strcasecmp(argv[optind], "unmarkauto")) ||
-	       (!strcasecmp(argv[optind], "forbid-version")) )
-	return cmdline_do_action(argc-optind, argv+optind,
-				 status_fname,
-				 simulate, assume_yes, download_only,
-				 fix_broken, showvers, showdeps, showsize,
-				 visual_preview, always_prompt, verbose);
-      else if(!strcasecmp(argv[optind], "upgrade"))
-	return cmdline_upgrade(argc-optind, argv+optind,
-			       status_fname,
-			       simulate, assume_yes, download_only,
-			       showvers, showdeps, showsize,
-			       visual_preview, always_prompt, verbose);
-      else if(!strcasecmp(argv[optind], "download"))
-	return cmdline_download(argc-optind, argv+optind);
-      else if(!strcasecmp(argv[optind], "changelog"))
-	return cmdline_changelog(argc-optind, argv+optind);
-      else if(!strcasecmp(argv[optind], "moo"))
-	return cmdline_moo(argc-optind, argv+optind, verbose);
-      else if(!strcasecmp(argv[optind], "show"))
-	return cmdline_show(argc-optind, argv+optind, verbose);
-      else if(!strcasecmp(argv[optind], "help"))
-	{
-	  usage();
-	  exit(0);
+	  // Read the terminal width and set up its update function.
+	  signal(SIGWINCH, update_screen_width);
+	  update_screen_width();
+
+	  if(update_only || install_only)
+	    {
+	      fprintf(stderr, "%s",
+		      _("-u and -i may not be specified with a command"));
+	      usage();
+	      exit(1);
+	    }
+
+	  if(!strcasecmp(argv[optind], "update"))
+	    return cmdline_update(argc-optind, argv+optind);
+	  else if(!strcasecmp(argv[optind], "clean"))
+	    return cmdline_clean(argc-optind, argv+optind, simulate);
+	  else if(!strcasecmp(argv[optind], "autoclean"))
+	    return cmdline_autoclean(argc-optind, argv+optind, simulate);
+	  else if(!strcasecmp(argv[optind], "forget-new"))
+	    return cmdline_forget_new(argc-optind, argv+optind,
+				      status_fname, simulate);
+	  else if(!strcasecmp(argv[optind], "search"))
+	    return cmdline_search(argc-optind, argv+optind,
+				  status_fname,
+				  display_format, width,
+				  sort_policy);
+	  else if( (!strcasecmp(argv[optind], "install")) ||
+		   (!strcasecmp(argv[optind], "reinstall")) ||
+		   (!strcasecmp(argv[optind], "dist-upgrade")) ||
+		   (!strcasecmp(argv[optind], "remove")) ||
+		   (!strcasecmp(argv[optind], "purge")) ||
+		   (!strcasecmp(argv[optind], "hold")) ||
+		   (!strcasecmp(argv[optind], "unhold")) ||
+		   (!strcasecmp(argv[optind], "markauto")) ||
+		   (!strcasecmp(argv[optind], "unmarkauto")) ||
+		   (!strcasecmp(argv[optind], "forbid-version")) ||
+		   (!strcasecmp(argv[optind], "keep")) ||
+		   (!strcasecmp(argv[optind], "keep-all")) )
+	    return cmdline_do_action(argc-optind, argv+optind,
+				     status_fname,
+				     simulate, assume_yes, download_only,
+				     fix_broken, showvers, showdeps, showsize,
+				     visual_preview, always_prompt,
+				     queue_only, verbose);
+	  else if(!strcasecmp(argv[optind], "upgrade"))
+	    return cmdline_upgrade(argc-optind, argv+optind,
+				   status_fname,
+				   simulate, assume_yes, download_only,
+				   showvers, showdeps, showsize,
+				   visual_preview, always_prompt,
+				   queue_only, verbose);
+	  else if(!strcasecmp(argv[optind], "download"))
+	    return cmdline_download(argc-optind, argv+optind);
+	  else if(!strcasecmp(argv[optind], "changelog"))
+	    return cmdline_changelog(argc-optind, argv+optind);
+	  else if(!strcasecmp(argv[optind], "moo"))
+	    return cmdline_moo(argc-optind, argv+optind, verbose);
+	  else if(!strcasecmp(argv[optind], "show"))
+	    return cmdline_show(argc-optind, argv+optind, verbose);
+	  else if(!strcasecmp(argv[optind], "dump-resolver"))
+	    return cmdline_dump_resolver(argc-optind, argv+optind, status_fname);
+	  else if(!strcasecmp(argv[optind], "help"))
+	    {
+	      usage();
+	      exit(0);
+	    }
+	  // Debugging/profiling commands:
+	  else if(!strcasecmp(argv[optind], "nop"))
+	    {
+	      OpTextProgress p(aptcfg->FindI("Quiet", 0));
+	      _error->DumpErrors();
+	      apt_init(&p, true);
+	      exit(0);
+	    }
+	  else if(!strcasecmp(argv[optind], "nop-noselections"))
+	    {
+	      OpTextProgress p(aptcfg->FindI("Quiet", 0));
+	      _error->DumpErrors();
+	      apt_init(&p, false);
+	      exit(0);
+	    }
+	  else
+	    {
+	      fprintf(stderr, _("Unknown command \"%s\"\n"), argv[optind]);
+	      usage();
+	      exit(1);
+	    }
 	}
-      else
+      catch(StdinEOFException)
 	{
-	  fprintf(stderr, _("Unknown command \"%s\"\n"), argv[optind]);
-	  usage();
-	  exit(1);
+	  printf("%s", _("Abort.\n"));
+	  return -1;
+	}
+      catch(const Exception &e)
+	{
+	  fprintf(stderr, _("Uncaught exception: %s\n"), e.errmsg().c_str());
+
+	  std::string backtrace = e.get_backtrace();
+	  if(!backtrace.empty())
+	    fprintf(stderr, _("Backtrace:\n%s"), backtrace.c_str());
+	  return -1;
 	}
     }
 
-  if(simulate)
-    {
-      fprintf(stderr, _("Invalid option for interactive mode: -s\n"));
-      usage();
-      exit(1);
-    }
-
-  // Apart from visual preview we also have to take care of the normal
-  // ui mess with UTF-8, and fallback to "C" locale
-  locale = setlocale(LC_ALL, NULL);    
-	      	      
-  if (strstr(locale, "UTF-8")) {
-    setlocale(LC_ALL, "C");
-    locale = setlocale(LC_ALL, NULL);
-  }			    	
-  
   ui_init();
 
-  vs_progress *p=gen_progress_bar();
-  apt_init(p, true, status_fname);
-  if(status_fname)
-    free(status_fname);
-  check_apt_errors();
-
-  file_quit.connect(SigC::slot(vscreen_exitmain));
-
-  if(apt_cache_file)
+  try
     {
-      (*apt_cache_file)->package_state_changed.connect(SigC::slot(do_vscreen_update));
-      (*apt_cache_file)->package_category_changed.connect(SigC::slot(vscreen_update));
+      vs_progress_ref p=gen_progress_bar();
+      apt_init(p.unsafe_get_ref(), true, status_fname);
+      if(status_fname)
+	free(status_fname);
+      check_apt_errors();
+
+      file_quit.connect(sigc::ptr_fun(vscreen_exitmain));
+
+      if(apt_cache_file)
+	{
+	  (*apt_cache_file)->package_state_changed.connect(sigc::ptr_fun(vscreen_update));
+	  (*apt_cache_file)->package_category_changed.connect(sigc::ptr_fun(vscreen_update));
+	}
+
+      do_new_package_view(*p.unsafe_get_ref());
+      p->destroy();
+      p = NULL;
+
+      if(update_only)
+	do_update_lists();
+      else if(install_only)
+	do_package_run_or_show_preview();
+      //install_or_remove_packages();
+
+      ui_main();
     }
+  catch(const Exception &e)
+    {
+      fprintf(stderr, _("Uncaught exception: %s\n"), e.errmsg().c_str());
 
-  do_new_package_view(*p);
-  p->destroy();
-
-  if(update_only)
-    do_update_lists();
-  else if(install_only)
-    do_package_run_or_show_preview();
-    //install_or_remove_packages();
-
-  ui_main();
+      std::string backtrace = e.get_backtrace();
+      if(!backtrace.empty())
+	fprintf(stderr, _("Backtrace:\n%s"), backtrace.c_str());
+      return -1;
+    }
 
   return 0;
 }
