@@ -2,7 +2,7 @@
 //
 //  The pkg_columnizer class.
 //
-//  Copyright 1999,2000,2001, 2005 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,25 +23,23 @@
 
 #include "pkg_columnizer.h"
 
-#include <generic/apt.h>
-#include <generic/config_signal.h>
-#include <generic/tasks.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/tasks.h>
 
 #include <apt-pkg/error.h>
-#include <apt-pkg/init.h> // For the APT version.
+#include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/policy.h>
 #include <apt-pkg/strutl.h>
 #include <apt-pkg/version.h>
 
+#include <vscreen/transcode.h>
+
 #include <unistd.h>
 
 column_definition_list *pkg_item::pkg_columnizer::columns=NULL;
-#if APT_PKG_MAJOR >=3 && APT_PKG_MINOR >= 5
 const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M%S %p %Z %v %V";
-#else
-const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M %p %Z %v %V";
-#endif
 
 // NOTE: the default widths here will be overridden by the initialization
 //      code.
@@ -72,9 +70,7 @@ column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columnizer::numtypes
   {15, false, true},    // downloadsize
   {4, false, false},    // pin_priority
   {15, false, true},    // hostname
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
-  {1, false, false},    // trust_state
-#endif
+  {1, false, false}     // trust_state
 };
 
 // Default widths for:
@@ -294,7 +290,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	  return column_disposition(_("hold"), 0);
 	else if(state.Upgradable() && !pkg.CurrentVer().end() &&
 		!candver.end() && candver.VerStr() == estate.forbidver)
-	  return column_disposition("forbidden upgrade", 0);
+	  return column_disposition(_("forbidden upgrade"), 0);
 	else if(state.Delete())
 	  return column_disposition((state.iFlags&pkgDepCache::Purge)?_("purge"):_("delete"), 0);
 	else if(state.InstBroken())
@@ -483,6 +479,7 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	buf[255]=0;
 	if(gethostname(buf, 255)!=0)
 	  // Panic
+	  // ForTranslators: Hostname
 	  return column_disposition(_("HN too long"), 0);
 	else
 	  return column_disposition(buf, 0);
@@ -518,7 +515,6 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(buf, 0);
       }
       break;
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
     case trust_state:
       {
 	if(visible_ver.end())
@@ -528,7 +524,6 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	else
 	  return column_disposition("U", 0);
       }
-#endif
     default:
       return column_disposition(_("ERROR"), 0);
     }
@@ -565,10 +560,8 @@ int pkg_item::pkg_columnizer::parse_column_type(char id)
       return priority;
     case 'R':
       return shortpriority;
-#if APT_PKG_MAJOR >=3 && APT_PKG_MINOR >= 5
     case 'S':
       return trust_state;
-#endif
     case 's':
       return section;
     case 'r':
@@ -651,28 +644,38 @@ void pkg_item::pkg_columnizer::setup_columns(bool force_update)
     delete columns;
   if(force_update || !columns)
     {
-      columns=parse_columns(aptcfg->Find(PACKAGE "::UI::Package-Display-Format", default_pkgdisplay),
-			    pkg_columnizer::parse_column_type,
-			    pkg_columnizer::defaults);
+      std::wstring cfg;
+
+      if(!transcode(aptcfg->Find(PACKAGE "::UI::Package-Display-Format",
+				 default_pkgdisplay).c_str(),
+		    cfg))
+	_error->Errno("iconv", _("Unable to transcode package display format after \"%ls\""), cfg.c_str());
+      else
+	columns=parse_columns(cfg,
+			      pkg_columnizer::parse_column_type,
+			      pkg_columnizer::defaults);
       if(!columns)
 	{
-	  columns=parse_columns(default_pkgdisplay,
-				pkg_columnizer::parse_column_type,
-				pkg_columnizer::defaults);
+	  cfg.clear();
+	  if(!transcode(default_pkgdisplay, cfg))
+	    _error->Errno("iconv", _("Unable to transcode package display format after \"%ls\""), cfg.c_str());
+	  else
+	    columns=parse_columns(cfg,
+				  pkg_columnizer::parse_column_type,
+				  pkg_columnizer::defaults);
 	  if(!columns)
 	    {
 	      _error->Warning(_("Internal error: Default column string is unparsable"));
-	      columns->push_back(column_definition(pkg_columnizer::name, pkg_columnizer::defaults[pkg_columnizer::name].width, true, true));
+	      columns->push_back(column_definition(column_definition::COLUMN_GENERATED, pkg_columnizer::name, pkg_columnizer::defaults[pkg_columnizer::name].width, true, true, false));
 	    }
 	}
     }
 }
 
-string pkg_item::pkg_columnizer::format_column_names(unsigned int width)
+std::wstring pkg_item::pkg_columnizer::format_column_names(unsigned int width)
 {
   setup_columns();
 
-  string rval=pkg_genheaders(*columns).layout_columns(width);
-
-  return rval;
+  empty_column_parameters p;
+  return pkg_genheaders(*columns).layout_columns(width, p);
 }
