@@ -27,18 +27,17 @@
 
 #include "vscreen/fragment.h"
 #include "vscreen/vs_layout_item.h"
-#include "vscreen/vs_staticitem.h"
 
-#include "desc_parse.h"
-#include "pkg_subtree.h"
-#include "pkg_item_with_subtree.h"
 #include "dep_item.h"
+#include "desc_parse.h"
+#include "pkg_item_with_subtree.h"
+#include "pkg_subtree.h"
 #include "pkg_ver_item.h"
 #include "trust.h"
 
-#include "generic/apt.h"
+#include <generic/apt/apt.h>
 
-#include <apt-pkg/init.h> // For the APT version.
+#include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/strutl.h>
 
 class pkg_grouppolicy_info:public pkg_grouppolicy
@@ -73,7 +72,6 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
 					      pkg_item_with_generic_subtree *tree,
 					      pkg_signal *sig)
 {
-
   char buf[256];
   
   if(!ver.end())
@@ -85,19 +83,21 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
 
       vector<fragment*> frags;
 
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
       fragment *untrusted_warning=make_untrusted_warning(ver);
       if(untrusted_warning != NULL)
 	{
 	  frags.push_back(untrusted_warning);
 	  frags.push_back(newline_fragment());
 	}
-#endif
 
       // Avoid creating new strings to translate.
       frags.push_back(clipbox(fragf("%B%s%b%s%n",
 				    _("Description: "), shortdesc.c_str())));
-      frags.push_back(indentbox(2, 2, make_desc_fragment(desc)));
+      frags.push_back(indentbox(2, 2, make_desc_fragment(transcode(desc))));
+
+      fragment *tags = make_tags_fragment(pkg);
+      if(tags != NULL)
+	frags.push_back(fragf("%n%F", tags));
 
       // Can I use something other than a clipbox below?
 
@@ -128,7 +128,7 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
       if(!ver.ProvidesList().end())
 	{
 	  snprintf(buf, 256, _("Package names provided by %s"), pkg.Name());
-	  pkg_subtree *prvtree=new pkg_subtree(buf);
+	  pkg_subtree *prvtree=new pkg_subtree(transcode(buf));
 
 	  for(pkgCache::PrvIterator prv=ver.ProvidesList(); !prv.end(); ++prv)
 	    prvtree->add_child(new pkg_item(prv.ParentPkg(), sig));
@@ -138,11 +138,11 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
     }
 
   snprintf(buf, 256, _("Packages which depend on %s"), pkg.Name());
-  pkg_subtree *revtree=new pkg_subtree(buf);
+  pkg_subtree *revtree=new pkg_subtree(transcode(buf));
   setup_package_deps<pkg_subtree>(pkg, ver, revtree, sig, true);
   tree->add_child(revtree);
 
-  pkg_vertree_generic *newtree=new pkg_vertree_generic(_("Versions"), true);
+  pkg_vertree_generic *newtree=new pkg_vertree_generic(transcode(_("Versions")), true);
   setup_package_versions(pkg, newtree, sig);
   tree->add_child(newtree);
 }
