@@ -1,6 +1,6 @@
 // pkg_item.cc
 //
-// Copyright 1999,2000,2001,2002 Daniel Burrows
+// Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,9 +23,10 @@
 
 #include <vscreen/config/column_definition.h>
 #include <vscreen/config/keybindings.h>
+#include <vscreen/fragment.h>
+#include <vscreen/transcode.h>
 #include <vscreen/vs_util.h>
 
-#include "download.h"
 #include "edit_pkg_hier.h"
 #include "pkg_columnizer.h"
 #include "pkg_item.h"
@@ -33,16 +34,17 @@
 #include "view_changelog.h"
 #include "vs_progress.h"
 
-#include <generic/apt.h>
-#include <generic/apt_undo_group.h>
-#include <generic/config_signal.h>
-#include <generic/matchers.h>
+#include <generic/apt/apt.h>
+#include <generic/apt/apt_undo_group.h>
+#include <generic/apt/config_signal.h>
+#include <generic/apt/matchers.h>
 
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
 
-#include <sigc++/bind.h>
-#include <sigc++/object_slot.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/functors/mem_fun.h>
+#include <sigc++/functors/ptr_fun.h>
 
 #include <iostream>
 
@@ -52,11 +54,11 @@ using namespace std;
 
 static const char *confirm_str=N_("Yes, I am aware this is a very bad idea");
 
-static void try_delete_essential(string s,
+static void try_delete_essential(wstring s,
 				 const pkgCache::PkgIterator pkg,
 				 bool purge)
 {
-  if(s==_(confirm_str))
+  if(s==transcode(_(confirm_str)))
     {
       undo_group *grp=new apt_undo_group;
 
@@ -75,16 +77,16 @@ static void confirm_delete_essential(const pkgCache::PkgIterator &pkg,
   assert((pkg->Flags&pkgCache::Flag::Essential)==pkgCache::Flag::Essential ||
 	 (pkg->Flags&pkgCache::Flag::Important)==pkgCache::Flag::Important);
 
-  char buf[512];
-  snprintf(buf, 512, _("%s is an essential package!\n\nAre you sure you want to remove it?\nType '%s' if you are."), pkg.Name(), _(confirm_str));
+  fragment *f=wrapbox(fragf(_("%s is an essential package!%n%nAre you sure you want to remove it?%nType '%s' if you are."), pkg.Name(), _(confirm_str)));
 
-  vscreen_widget *w=vs_dialog_string(buf,
-				     "",
-				     bind(SigC::slot(try_delete_essential),
-					  pkg, purge),
-				     NULL,
-				     NULL,
-				     NULL);
+  vs_widget_ref w=vs_dialog_string(f,
+				   L"",
+				   arg(sigc::bind(sigc::ptr_fun(try_delete_essential),
+						  pkg, purge)),
+				   NULL,
+				   NULL,
+				   NULL,
+				   style_attrs_flip(A_REVERSE));
 
   w->show_all();
 
@@ -92,7 +94,7 @@ static void confirm_delete_essential(const pkgCache::PkgIterator &pkg,
 }
 
 pkg_item::pkg_item(const pkgCache::PkgIterator &_package,
-		   SigC::Signal2<void, const pkgCache::PkgIterator &, const pkgCache::VerIterator &> *sig)
+		   sigc::signal2<void, const pkgCache::PkgIterator &, const pkgCache::VerIterator &> *sig)
 :package(_package), info_signal(sig)
 {
 }
@@ -151,7 +153,7 @@ void pkg_item::keep(undo_group *undo)
   // Keep, don't hold, the package.
   (*apt_cache_file)->mark_keep(package,
 			       false,
-			       (*apt_cache_file)->get_ext_state(package).selection_state==pkgCache::State::Hold,
+			       false,
 			       undo);
 }
 
@@ -216,10 +218,13 @@ void pkg_item::show_information()
 {
   char buf[512]="Foo";
   snprintf(buf, 512, _("Information about %s"), package.Name());
+  string menulabel(buf);
+  snprintf(buf, 512, _("%s info"), package.Name());
+  string tablabel(buf);
 
-  vscreen_widget *w=make_info_screen(package, visible_version());
+  vs_widget_ref w=make_info_screen(package, visible_version());
   // what to use as the menu description?
-  insert_main_widget(w, buf, "");
+  insert_main_widget(w, menulabel, "", tablabel);
 }
 
 void pkg_item::show_changelog()
@@ -227,12 +232,19 @@ void pkg_item::show_changelog()
    view_changelog(visible_version());
 }
 
-int pkg_item::get_normal_attr()
+style pkg_item::get_highlight_style()
 {
-  return pkg_attr(package, vs_treeitem::get_normal_attr());
+  return vs_treeitem::get_normal_style() + pkg_style(package, true);
 }
 
-int pkg_item::pkg_attr(pkgCache::PkgIterator package, int normal_attr)
+style pkg_item::get_normal_style()
+{
+  return vs_treeitem::get_normal_style() + pkg_style(package, false);
+}
+
+#define MAYBE_HIGHLIGHTED(x) (highlighted ? (x "Highlighted") : (x))
+
+style pkg_item::pkg_style(pkgCache::PkgIterator package, bool highlighted)
 {
   if(package.VersionList().end())
     {
@@ -251,13 +263,13 @@ int pkg_item::pkg_attr(pkgCache::PkgIterator package, int normal_attr)
 	}
 
       if(present_now && present_inst)
-	return A_BOLD|normal_attr;
+	return get_style(MAYBE_HIGHLIGHTED("PkgIsInstalled"));
       else if(present_now)
-	return get_color("PkgToRemove");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToRemove"));
       else if(present_inst)
-	return get_color("PkgToInstall");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToInstall"));
       else
-	return normal_attr;
+	return get_style(MAYBE_HIGHLIGHTED("PkgNotInstalled"));
     }
   else
     {
@@ -265,25 +277,25 @@ int pkg_item::pkg_attr(pkgCache::PkgIterator package, int normal_attr)
 
       if(!state.InstBroken() &&
 	 (state.NewInstall() || (state.iFlags&pkgDepCache::ReInstall)))
-	return get_color("PkgToInstall");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToInstall"));
       else if(state.Status!=2 && // Not being upgraded
 	      (*apt_cache_file)->get_ext_state(package).selection_state==pkgCache::State::Hold // Flagged for hold
 	      && !state.InstBroken()) // Not currently broken.
-	return get_color("PkgToHold");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToHold"));
       else if(state.Delete())
-	return get_color("PkgToRemove");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToRemove"));
       else if(state.InstBroken())
-	return get_color("PkgBroken");
+	return get_style(MAYBE_HIGHLIGHTED("PkgBroken"));
       else if(state.Upgrade())
-	return get_color("PkgToUpgrade");
+	return get_style(MAYBE_HIGHLIGHTED("PkgToUpgrade"));
       else if(package.CurrentVer().end())
-	return normal_attr;
+	return get_style(MAYBE_HIGHLIGHTED("PkgNotInstalled"));
       else
-	return A_BOLD|normal_attr;
+	return get_style(MAYBE_HIGHLIGHTED("PkgIsInstalled"));
     }
 }
 
-void pkg_item::paint(vs_tree *win, int y, bool hierarchical)
+void pkg_item::paint(vs_tree *win, int y, bool hierarchical, const style &st)
 {
   int basex=hierarchical?2*get_depth():0;
   int width, height;
@@ -291,44 +303,58 @@ void pkg_item::paint(vs_tree *win, int y, bool hierarchical)
   win->getmaxyx(height, width);
   pkg_columnizer::setup_columns();
 
-  string disp=pkg_columnizer(package, visible_version(), pkg_columnizer::get_columns(), basex).layout_columns(width);
+  empty_column_parameters p;
+  wstring disp=pkg_columnizer(package, visible_version(), pkg_columnizer::get_columns(), basex).layout_columns(width, p);
   win->mvaddnstr(y, 0, disp.c_str(), width);
 }
 
-bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
+bool pkg_item::dispatch_key(const key &k, vs_tree *owner)
 {
-  if(bindings->key_matches(ch, "Versions"))
+  if(bindings->key_matches(k, "Versions"))
     {
-      vscreen_widget *w=make_ver_screen(package);
-      insert_main_widget(w, string(_("Available versions of "))+package.Name(),
-			 "");
+      char buf[512];
+      snprintf(buf, 512, _("Available versions of %s"),
+	       package.Name());
+      string menulabel(buf);
+      snprintf(buf, 512, _("%s versions"),
+	       package.Name());
+      string tablabel(buf);
+
+      vs_widget_ref w=make_ver_screen(package);
+      insert_main_widget(w, menulabel, "", tablabel);
     }
-  else if(bindings->key_matches(ch, "Dependencies"))
+  else if(bindings->key_matches(k, "Dependencies"))
     {
       if(!visible_version().end())
 	{
 	  char buf[512];
 	  snprintf(buf, 512, _("Dependencies of %s"), package.Name());
+	  string menulabel(buf);
+	  snprintf(buf, 512, _("%s deps"), package.Name());
+	  string tablabel(buf);
 
-	  vscreen_widget *w=make_dep_screen(package, visible_version());
-	  insert_main_widget(w, buf, "");
+	  vs_widget_ref w=make_dep_screen(package, visible_version());
+	  insert_main_widget(w, menulabel, "", tablabel);
 	  w->show();
 	}
     }
-  else if(bindings->key_matches(ch, "ReverseDependencies"))
+  else if(bindings->key_matches(k, "ReverseDependencies"))
     {
       char buf[512];
       snprintf(buf, 512, _("Packages depending on %s"), package.Name());
+      string menulabel(buf);
+      snprintf(buf, 512, _("%s reverse deps"), package.Name());
+      string tablabel(buf);
 
-      vscreen_widget *w=make_dep_screen(package, visible_version(), true);
-      insert_main_widget(w, buf, "");
+      vs_widget_ref w=make_dep_screen(package, visible_version(), true);
+      insert_main_widget(w, menulabel, "", tablabel);
     }
-  else if(bindings->key_matches(ch, "InfoScreen"))
+  else if(bindings->key_matches(k, "InfoScreen"))
     show_information();
-  else if(bindings->key_matches(ch, "Changelog") &&
+  else if(bindings->key_matches(k, "Changelog") &&
 	  !visible_version().end())
     show_changelog();
-  else if(bindings->key_matches(ch, "InstallSingle"))
+  else if(bindings->key_matches(k, "InstallSingle"))
     {
       if((*apt_cache_file)[package].CandidateVerIter(*apt_cache_file).end())
 	return true;
@@ -340,7 +366,7 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       else
 	delete grp;
     }
-  else if(bindings->key_matches(ch, "ForbidUpgrade"))
+  else if(bindings->key_matches(k, "ForbidUpgrade"))
     {
       undo_group *grp=new apt_undo_group;
       forbid_upgrade(grp);
@@ -350,7 +376,7 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       else
 	delete grp;
     }
-  else if(bindings->key_matches(ch, "BugReport"))
+  else if(bindings->key_matches(k, "BugReport"))
     {
       // Try to report a bug on the package.  (ew quoting ew)
       string cmd=string("reportbug '")+package.Name()+"'";
@@ -367,17 +393,13 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
 
       vscreen_suspend();
 
-      apt_cache_file->ReleaseLock();
-
       printf(_("Reporting a bug in %s:\n"), package.Name());
 
       system(cmd.c_str());
 
       vscreen_resume();
-
-      apt_reload_cache(gen_progress_bar(), true);
     }
-  else if(bindings->key_matches(ch, "DpkgReconfigure"))
+  else if(bindings->key_matches(k, "DpkgReconfigure"))
     // Don't bother with my internal su-to-root stuff here, since I don't
     // need to touch the package lists in the subprocess.
     {
@@ -391,7 +413,7 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       else if(access("/bin/su", X_OK)==0)
 	sucmd="/bin/su -c \"/usr/sbin/dpkg-reconfigure '%s'\"";
       else
-	popup_widget(vs_dialog_ok(_("You are not root and I cannot find any way\nto become root.  To reconfigure this package,\ninstall the menu package, the login package,\nor run aptitude as root.")));
+	popup_widget(vs_dialog_ok(text_fragment(_("You are not root and I cannot find any way to become root.  To reconfigure this package, install the menu package, the login package, or run aptitude as root."))));
 
       if(sucmd)
 	{
@@ -415,22 +437,24 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
 	      vscreen_resume();
 	    }
 
-	  apt_reload_cache(gen_progress_bar(), true);
+	  vs_progress_ref p = gen_progress_bar();
+	  apt_reload_cache(p.unsafe_get_ref(), true);
+	  p->destroy();
 	}
     }
-  else if(bindings->key_matches(ch, "EditHier"))
+  else if(bindings->key_matches(k, "EditHier"))
     {
-      vs_hier_editor *e=new vs_hier_editor;
+      vs_hier_editor_ref e=vs_hier_editor::create();
       e->set_package(package, visible_version());
 
       // FIXME: better title
-      add_main_widget(e, _("Hierarchy editor"), "");
+      add_main_widget(e, _("Hierarchy editor"), "", _("Hierarchy Editor"));
 
       e->connect_key("Quit", &global_bindings,
-		     slot(*e, &vscreen_widget::destroy));
+		     sigc::mem_fun(*e.unsafe_get_ref(), &vscreen_widget::destroy));
     }
   else
-    return pkg_tree_node::dispatch_char(ch, owner);
+    return pkg_tree_node::dispatch_key(k, owner);
 
   return true;
 }
@@ -444,7 +468,7 @@ void pkg_item::dispatch_mouse(short id, int x, mmask_t bstate, vs_tree *owner)
     show_information();
 }
 
-bool pkg_item::matches(string s) const
+bool pkg_item::matches(const string &s) const
 {
   return pkg_matches(s, package, visible_version());
 }
@@ -464,14 +488,24 @@ pkgCache::VerIterator pkg_item::visible_version(const pkgCache::PkgIterator &pkg
     return pkg.CurrentVer();
 }
 
-const char *pkg_item::tag() const
+const wchar_t *pkg_item::tag()
 {
-  return package.Name();
+  // FIXME: ew
+  static wstring pkgname;
+
+  pkgname=transcode(package.Name(), "ASCII");
+
+  return pkgname.c_str();
 }
 
-const char *pkg_item::label() const
+const wchar_t *pkg_item::label()
 {
-  return package.Name();
+  // FIXME: ew
+  static wstring pkgname;
+
+  pkgname=transcode(package.Name(), "ASCII");
+
+  return pkgname.c_str();
 }
 
 const pkgCache::PkgIterator &pkg_item::get_package() const
@@ -483,3 +517,48 @@ pkgCache::VerIterator pkg_item::visible_version() const
 {
   return visible_version(package);
 }
+
+//////////////////////// Menu redirections: /////////////////////////////
+
+bool pkg_item::package_forbid_enabled()
+{
+  return true;
+}
+
+bool pkg_item::package_forbid()
+{
+  undo_group *grp=new apt_undo_group;
+
+  forbid_upgrade(grp);
+
+  if(!grp->empty())
+    apt_undos->add_item(grp);
+  else
+    delete grp;
+
+  package_states_changed();
+
+  return true;
+}
+
+bool pkg_item::package_changelog_enabled()
+{
+  return true;
+}
+
+bool pkg_item::package_changelog()
+{
+  show_changelog();
+  return true;
+}
+
+bool pkg_item::package_information_enabled()
+{
+  return true;
+}
+
+bool pkg_item::package_information()
+{
+  show_information();
+  return true;
+}
