@@ -1,6 +1,6 @@
 // pkg_item.h  -*-c++-*-
 //
-//  Copyright 1999,2000 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ class pkg_item:public pkg_tree_node
 {
   pkgCache::PkgIterator package;
 
-  SigC::Signal2<void,
+  sigc::signal2<void,
 		const pkgCache::PkgIterator &,
 		const pkgCache::VerIterator &> *info_signal;
   // To be called when we're highlighted or unhighlighted..
@@ -47,15 +47,15 @@ public:
   class pkg_columnizer;
   static pkgCache::VerIterator visible_version(const pkgCache::PkgIterator &pkg);
   pkg_item(const pkgCache::PkgIterator &_package,
-	   SigC::Signal2<void,
+	   sigc::signal2<void,
 	   const pkgCache::PkgIterator &,
 	   const pkgCache::VerIterator &> *sig);
 
-  virtual void paint(vs_tree *win, int y, bool hierarchical);
+  virtual void paint(vs_tree *win, int y, bool hierarchical, const style &st);
 
-  virtual const char *tag() const;
-  virtual const char *label() const;
-  virtual bool matches(std::string s) const;
+  virtual const wchar_t *tag();
+  virtual const wchar_t *label();
+  virtual bool matches(const std::string &s) const;
 
   virtual void select(undo_group *undo);
   virtual void hold(undo_group *undo);
@@ -69,7 +69,8 @@ public:
   virtual void highlighted(vs_tree *win);
   virtual void unhighlighted(vs_tree *win);
 
-  virtual int get_normal_attr();
+  virtual style get_highlight_style();
+  virtual style get_normal_style();
 
   void show_changelog();
   void show_information();
@@ -77,14 +78,24 @@ public:
   const pkgCache::PkgIterator &get_package() const;
   pkgCache::VerIterator visible_version() const;
 
-  bool dispatch_char(chtype ch, vs_tree *owner);
+  bool dispatch_key(const key &k, vs_tree *owner);
   void dispatch_mouse(short id, int x, mmask_t bstate, vs_tree *owner);
 
-  /** Returns the attributes that would be used to display the given
-   *  package.  normal_attr indicates the "background" attributes that
-   *  should be used.
+  /** Returns the style that would be used to display the given
+   *  package.
+   *
+   *  \param package the package to generate a style for
+   *  \param highlighted if \b true, the package is highlighted
    */
-  static int pkg_attr(pkgCache::PkgIterator package, int normal_attr);
+  static style pkg_style(pkgCache::PkgIterator package, bool highlighted);
+
+  // Menu redirections:
+  bool package_forbid_enabled();
+  bool package_forbid();
+  bool package_changelog_enabled();
+  bool package_changelog();
+  bool package_information_enabled();
+  bool package_information();
 };
 
 #endif
