@@ -1,6 +1,6 @@
 // pkg_node.h	-*-c++-*-
 //
-//  Copyright 1999 Daniel Burrows
+//  Copyright 1999-2000, 2002, 2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,15 +23,20 @@
 #ifndef PKG_NODE_H
 #define PKG_NODE_H
 
+#include "menu_redirect.h"
+
 #include <vscreen/vs_treeitem.h>
-#include <generic/undo.h>
 
 class undo_group;
 class keybindings;
 
-class pkg_tree_node:virtual public vs_treeitem
+class pkg_tree_node:virtual public vs_treeitem, public menu_redirect
 // Provides some extra package-related interfaces.
 {
+  /** Used to convert calls via the menu interface to wrapped
+   *  invocations of the low-level action methods.
+   */
+  bool package_action(void (pkg_tree_node::* action)(undo_group *));
 public:
   virtual void select(undo_group *undo)=0;
   virtual void hold(undo_group *undo)=0;
@@ -45,12 +50,22 @@ public:
   void mark_auto(undo_group *undo) {set_auto(true, undo);}
   void unmark_auto(undo_group *undo) {set_auto(false, undo);}
 
-  bool dispatch_char(chtype ch, vs_tree *owner);
+  bool dispatch_key(const key &k, vs_tree *owner);
   // IMPORTANT NOTE: pkg_tree_node::dispatch_char() does NOT call
   // vs_treeitem::dispatch_char!
 
   static keybindings *bindings;
   static void init_bindings();
+
+  // Menu redirections:
+  bool package_enabled();
+  bool package_install();
+  bool package_remove();
+  bool package_purge();
+  bool package_hold();
+  bool package_keep();
+  bool package_mark_auto();
+  bool package_unmark_auto();
 };
 
 #endif
