@@ -1,6 +1,6 @@
 // pkg_ver_item.cc
 //
-//  Copyright 1999,2000,2001, 2005 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -27,13 +27,14 @@
 #include "pkg_item.h"
 #include "pkg_subtree.h"
 #include "pkg_ver_item.h"
+#include "pkg_sortpolicy.h"
 #include "ui.h"
 #include "view_changelog.h"
 #include "vs_progress.h"
 
-#include "generic/apt.h"
-#include "generic/apt_undo_group.h"
-#include "generic/config_signal.h"
+#include <generic/apt/apt.h>
+#include <generic/apt/apt_undo_group.h>
+#include <generic/apt/config_signal.h>
 
 #include "vscreen/vs_multiplex.h"
 
@@ -41,10 +42,12 @@
 #include <string>
 
 #include <apt-pkg/configuration.h>
-#include <apt-pkg/init.h> // For the APT version
+#include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/policy.h>
 #include <apt-pkg/strutl.h>
 
+#include <signal.h>
+
 using namespace std;
 
 class pkg_ver_columnizer:public pkg_item::pkg_columnizer
@@ -350,14 +353,19 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
       switch(ver->Priority)
 	{
 	case pkgCache::State::Important:
+	  // ForTranslators: Imp = Important
 	  return column_disposition(_("Imp"), 0);
 	case pkgCache::State::Required:
+	  // ForTranslators: Req = Required
 	  return column_disposition(_("Req"), 0);
 	case pkgCache::State::Standard:
+	  // ForTranslators: Std = Standard
 	  return column_disposition(_("Std"), 0);
 	case pkgCache::State::Optional:
+	  // ForTranslators: Opt = Optional
 	  return column_disposition(_("Opt"), 0);
 	case pkgCache::State::Extra:
+	  // ForTranslators: Xtr = Extra
 	  return column_disposition(_("Xtr"), 0);
 	default:
 	  return column_disposition(_("ERR"), 0);
@@ -403,7 +411,6 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	snprintf(buf, 100, "%i", count);
 	return column_disposition(buf, 0);
       }
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
     case trust_state:
       {
 	if(ver.end())
@@ -413,7 +420,6 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	else
 	  return column_disposition("U", 0);
       }
-#endif
     default:
       return pkg_columnizer::setup_column(type);
     }
@@ -435,6 +441,10 @@ void setup_package_versions(const pkgCache::PkgIterator &pkg, pkg_vertree *tree,
 
   for(pkgCache::PrvIterator i=pkg.ProvidesList(); !i.end(); i++)
     tree->add_child(new pkg_ver_item(i.OwnerVer(), sig, true));
+
+  std::auto_ptr<pkg_sortpolicy> sorter(pkg_sortpolicy_ver(0, false));
+  pkg_sortpolicy_wrapper wrap(sorter.get());
+  tree->sort(wrap);
 }
 
 void setup_package_versions(const pkgCache::PkgIterator &pkg, pkg_vertree_generic *tree, pkg_signal *sig)
@@ -444,6 +454,10 @@ void setup_package_versions(const pkgCache::PkgIterator &pkg, pkg_vertree_generi
 
   for(pkgCache::PrvIterator i=pkg.ProvidesList(); !i.end(); i++)
     tree->add_child(new pkg_ver_item(i.OwnerVer(), sig, true));
+
+  std::auto_ptr<pkg_sortpolicy> sorter(pkg_sortpolicy_ver(0, false));
+  pkg_sortpolicy_wrapper wrap(sorter.get());
+  tree->sort(wrap);
 }
 
 void pkg_grouppolicy_ver::add_package(const pkgCache::PkgIterator &pkg,
@@ -462,12 +476,27 @@ pkg_grouppolicy *pkg_grouppolicy_ver_factory::instantiate(pkg_signal *sig,
   return new pkg_grouppolicy_ver(sig, desc_sig);
 }
 
-int pkg_ver_item::get_normal_attr()
+style pkg_ver_item::get_normal_style()
 {
-  return ver_attr(version, vs_treeitem::get_normal_attr());
+  return vs_treeitem::get_normal_style() + ver_style(version, false);
 }
 
-int pkg_ver_item::ver_attr(pkgCache::VerIterator version, int normal_attr)
+style pkg_ver_item::get_highlighted_style()
+{
+  return vs_treeitem::get_normal_style() + ver_style(version, true);
+}
+
+pkg_ver_item::pkg_ver_item(const pkgCache::VerIterator &_version, pkg_signal *_sig,
+			   bool _show_pkg_name)
+  :version(_version), version_name(transcode(version.VerStr(), "ASCII")),
+   show_pkg_name(_show_pkg_name), sig(_sig)
+{
+}
+
+#define MAYBE_HIGHLIGHTED(x) (highlighted ? (x "Highlighted") : (x))
+
+style pkg_ver_item::ver_style(pkgCache::VerIterator version,
+			      bool highlighted)
 {
   pkgCache::PkgIterator pkg=version.ParentPkg();
   pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
@@ -477,29 +506,30 @@ int pkg_ver_item::ver_attr(pkgCache::VerIterator version, int normal_attr)
       (state.Install() &&
        !state.InstBroken() &&
        state.InstVerIter(*apt_cache_file)!=version)))
-    return get_color("PkgToRemove");
+    return get_style(MAYBE_HIGHLIGHTED("PkgToRemove"));
 
   else if(((state.NewInstall() || state.Install()) &&
 	   !state.InstBroken() &&
 	   state.InstVerIter(*apt_cache_file)==version) ||
 	  (version==version.ParentPkg().CurrentVer() &&
 	   state.iFlags&pkgDepCache::ReInstall))
-    return get_color("PkgToInstall");
+    return get_style(MAYBE_HIGHLIGHTED("PkgToInstall"));
 
   else if(state.InstBroken() && state.InstVerIter(*apt_cache_file)==version)
-    return get_color("PkgBroken");
+    return get_style(MAYBE_HIGHLIGHTED("PkgBroken"));
 
   else if(pkg.CurrentVer()!=version)
-    return normal_attr;
+    return get_style(MAYBE_HIGHLIGHTED("PkgNotInstalled"));
 
   else if(state.NowBroken())
-    return get_color("PkgBroken");
+    return get_style(MAYBE_HIGHLIGHTED("PkgBroken"));
 
   else
-    return A_BOLD|normal_attr;
+    return get_style(MAYBE_HIGHLIGHTED("PkgIsInstalled"));
 }
 
-void pkg_ver_item::paint(vs_tree *win, int y, bool hierarchical)
+void pkg_ver_item::paint(vs_tree *win, int y, bool hierarchical,
+			 const style &st)
 {
   int basex=hierarchical?2*get_depth():0;
   int width, height;
@@ -507,10 +537,11 @@ void pkg_ver_item::paint(vs_tree *win, int y, bool hierarchical)
   win->getmaxyx(height, width);
   pkg_item::pkg_columnizer::setup_columns();
 
-  string disp=pkg_ver_columnizer(version,
-				 show_pkg_name,
-				 pkg_item::pkg_columnizer::get_columns(),
-				 basex).layout_columns(width);
+  empty_column_parameters p;
+  wstring disp=pkg_ver_columnizer(version,
+				  show_pkg_name,
+				  pkg_item::pkg_columnizer::get_columns(),
+				  basex).layout_columns(width, p);
   win->mvaddnstr(y, 0, disp.c_str(), width);
 }
 
@@ -527,14 +558,14 @@ void pkg_ver_item::unhighlighted(vs_tree *win)
 	   pkgCache::VerIterator(*apt_cache_file));
 }
 
-const char *pkg_ver_item::tag() const
+const wchar_t *pkg_ver_item::tag()
 {
-  return version.VerStr();
+  return version_name.c_str();
 }
 
-const char *pkg_ver_item::label() const
+const wchar_t *pkg_ver_item::label()
 {
-  return version.VerStr();
+  return version_name.c_str();
 }
 
 void pkg_ver_item::select(undo_group *undo)
@@ -621,8 +652,15 @@ void pkg_ver_item::forbid_version(undo_group *undo)
 
 void pkg_ver_item::show_information()
 {
-  vscreen_widget *w=make_info_screen(version.ParentPkg(), version);
-  insert_main_widget(w, string(_("Information about "))+version.ParentPkg().Name(), "");
+  vs_widget_ref w=make_info_screen(version.ParentPkg(), version);
+
+  char buf[512];
+  snprintf(buf, 512, _("Information about %s"), version.ParentPkg().Name());
+  string menulabel(buf);
+  snprintf(buf, 512, _("%s info"), version.ParentPkg().Name());
+  string tablabel(buf);
+
+  insert_main_widget(w, menulabel, "", tablabel);
 }
 
 pkg_ver_screen::pkg_ver_screen(const pkgCache::PkgIterator &pkg)
@@ -642,37 +680,43 @@ vs_treeitem *pkg_ver_screen::setup_new_root(const pkgCache::PkgIterator &pkg,
   return newtree;
 }
 
-bool pkg_ver_item::dispatch_char(chtype ch, vs_tree *owner)
+bool pkg_ver_item::dispatch_key(const key &k, vs_tree *owner)
 {
-  if(bindings->key_matches(ch, "Dependencies"))
+  if(bindings->key_matches(k, "Dependencies"))
     {
       char buf[512];
       snprintf(buf, 512, _("Dependencies of %s"), version.ParentPkg().Name());
+      string menulabel(buf);
+      snprintf(buf, 512, _("%s deps"), version.ParentPkg().Name());
+      string tablabel(buf);
 
-      vscreen_widget *w=make_dep_screen(version.ParentPkg(), version);
-      insert_main_widget(w, buf, "");
+      vs_widget_ref w=make_dep_screen(version.ParentPkg(), version);
+      insert_main_widget(w, menulabel, "", tablabel);
       return true;
     }
-  else if(bindings->key_matches(ch, "ReverseDependencies"))
+  else if(bindings->key_matches(k, "ReverseDependencies"))
     {
       char buf[512];
       snprintf(buf, 512, _("Packages depending on %s"), version.ParentPkg().Name());
+      string menulabel(buf);
+      snprintf(buf, 512, _("%s reverse deps"), version.ParentPkg().Name());
+      string tablabel(buf);
 
-      vscreen_widget *w=make_dep_screen(version.ParentPkg(), version, true);
-      insert_main_widget(w, buf, "");
+      vs_widget_ref w=make_dep_screen(version.ParentPkg(), version, true);
+      insert_main_widget(w, menulabel, "", tablabel);
       return true;
     }
-  else if(bindings->key_matches(ch, "InfoScreen"))
+  else if(bindings->key_matches(k, "InfoScreen"))
     {
       show_information();
       return true;
     }
-  else if(bindings->key_matches(ch, "Changelog"))
+  else if(bindings->key_matches(k, "Changelog"))
     {
       view_changelog(version);
       return true;
     }
-  else if(bindings->key_matches(ch, "ForbidUpgrade"))
+  else if(bindings->key_matches(k, "ForbidUpgrade"))
     {
       undo_group *grp=new apt_undo_group;
       forbid_version(grp);
@@ -684,25 +728,91 @@ bool pkg_ver_item::dispatch_char(chtype ch, vs_tree *owner)
 
       return true;
     }
-  else if(bindings->key_matches(ch, "BugReport"))
+  else if(bindings->key_matches(k, "BugReport"))
     {
       // Try to report a bug on the package.  (ew quoting ew)
       string cmd=string("reportbug '")+version.ParentPkg().Name()+"' -V '"+version.VerStr()+"'";
 
+
+      struct sigaction oldact;
+      struct sigaction act;
+      act.sa_handler = SIG_DFL;
+      act.sa_sigaction = 0;
+      sigemptyset(&act.sa_mask);
+      act.sa_flags = 0;
+      act.sa_restorer = 0;
+
+      sigaction(SIGCONT, &act, &oldact);
+
       vscreen_suspend();
 
       apt_cache_file->ReleaseLock();
 
       printf(_("Reporting a bug in %s:\n"), version.ParentPkg().Name());
 
+
+
+
       system(cmd.c_str());
 
+      sigaction(SIGCONT, &oldact, NULL);
+
+
+
       vscreen_resume();
 
-      apt_reload_cache(gen_progress_bar(), true);
+      vs_progress_ref p = gen_progress_bar();
+      apt_reload_cache(p.unsafe_get_ref(), true);
+      p->destroy();
 
       return true;
     }
   else
-    return pkg_tree_node::dispatch_char(ch, owner);
+    return pkg_tree_node::dispatch_key(k, owner);
 }
+
+
+////////////////////////// Menu redirections: ////////////////////////////
+
+bool pkg_ver_item::package_forbid_enabled()
+{
+  return true;
+}
+
+bool pkg_ver_item::package_forbid()
+{
+  undo_group *grp = new apt_undo_group;
+
+  forbid_version(grp);
+
+  if(!grp->empty())
+    apt_undos->add_item(grp);
+  else
+    delete grp;
+
+  package_states_changed();
+  return true;
+}
+
+bool pkg_ver_item::package_changelog_enabled()
+{
+  return true;
+}
+
+bool pkg_ver_item::package_changelog()
+{
+  view_changelog(get_version());
+  return true;
+}
+
+bool pkg_ver_item::package_information_enabled()
+{
+  return true;
+}
+
+bool pkg_ver_item::package_information()
+{
+  show_information();
+  return true;
+}
+
