@@ -1,13 +1,9 @@
-#include <apt-pkg/init.h> // For the APT version.
-
-#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
-
 // trust.cc
 
 #include "aptitude.h"
 #include "trust.h"
 
-#include <generic/apt.h>
+#include <generic/apt/apt.h>
 
 #include <vscreen/fragment.h>
 #include <vscreen/config/colors.h>
@@ -21,8 +17,6 @@ fragment *make_untrusted_warning(const pkgCache::VerIterator &ver)
 		     strlen(_("WARNING"))+2,
 		     flowbox(fragf(_("%F: This version of %s is from an %Buntrusted source%b!  Installing this package could allow a malicious individual to damage or take control of your system."),
 				   text_fragment(_("WARNING"),
-						 get_color("TrustWarning")),
+						 get_style("TrustWarning")),
 				   ver.ParentPkg().Name())));
 }
-
-#endif
