@@ -1,16 +1,16 @@
 // vs_progress.cc
 //
-//  Copyright 2000 Daniel Burrows
+//  Copyright 2000, 2004-2005 Daniel Burrows
 //
-//  This program is free software; you can redistribute it and/or modify
-//  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
-//  (at your option) any later version.
+//  This program is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU General Public License as
+//  published by the Free Software Foundation; either version 2 of the
+//  License, or (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; see the file COPYING.  If not, write to
@@ -23,7 +23,13 @@
 
 #include "vs_progress.h"
 
-void vs_progress::paint()
+#include <vscreen/transcode.h>
+
+vs_progress::vs_progress()
+{
+}
+
+void vs_progress::paint(const style &st)
 {
   int barsize;
   char percent_string[50];
@@ -38,17 +44,14 @@ void vs_progress::paint()
 
       show_string_as_progbar(0,
 			     0,
-			     Op+percent_string,
-			     get_color("Progress"),
-			     get_color("ScreenStatusColor"),
+			     transcode(Op+percent_string),
+			     st+get_style("Progress"),
+			     st,
 			     barsize,
 			     width);
     }
   else
-    {
-      bkgd(get_color("ScreenStatusColor"));
-      erase();
-    }
+    erase();
 }
 
 bool vs_progress::get_cursorvisible()
@@ -63,6 +66,8 @@ point vs_progress::get_cursorloc()
 
 void vs_progress::Update()
 {
+  show();
+
   if(CheckChange(0.25))
     {
       vscreen_update();
@@ -76,6 +81,15 @@ void vs_progress::Done()
   vscreen_update();
   vscreen_updatecursor();
   vscreen_tryupdate();
-
   hide();
 }
+
+int vs_progress::width_request()
+{
+  return 40;
+}
+
+int vs_progress::height_request(int w)
+{
+  return 1;
+}
