@@ -1,6 +1,6 @@
 // columnify.h    -*-c++-*-
 //
-//  Copyright 2000 Daniel Burrows
+//  Copyright 2000, 2005 Daniel Burrows
 //  
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -39,17 +39,21 @@
 
 struct column_disposition
 {
-  std::string text;  // The contents of the columns
+  std::wstring text;  // The contents of the columns
   int minx;          // The minimum x value to start this column at (useful for
                      // indenting stuff in trees)
 
-  column_disposition(std::string _text, int _minx):text(_text), minx(_minx) {}
+  column_disposition(const std::wstring &_text, int _minx):text(_text), minx(_minx) {}
+
+  /** Generate a column from the multibyte string _text. */
+  column_disposition(const std::string &_text, int _minx,
+		     const char *encoding=NULL);
 };
 
 struct column
 {
   column_disposition info;
-  unsigned int width;
+  int width;
   bool expand, shrink;
 
   column(const column_disposition &_info, int _width, bool _expand, bool _shrink)
@@ -63,10 +67,11 @@ typedef std::list<column> column_list;
 
 typedef std::list<column> layout;
 
-std::string columnify(const layout &format, unsigned int width);
-// Returns a string formatted as requested.  The string will be no longer than
-// width characters.
-// I could probably use printf-style strings, but with the groups it would
-// probably turn into a major pain..
+/* \return a string formatted as requested.  The string will be no
+ * wider than width columns.  I could probably use printf-style
+ * strings, but with the groups it would probably turn into a major
+ * pain..
+ */
+std::wstring columnify(const layout &format, int width);
 
 #endif
