@@ -1,6 +1,6 @@
 // column_definition.h   -*-c++-*-
 //
-//  Copyright 2000 Daniel Burrows
+//  Copyright 2000, 2005 Daniel Burrows
 //  
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -43,26 +43,69 @@
 
 #include <assert.h>
 
-#include "../columnify.h"
+#include <vscreen/columnify.h>
 
+/** Defined the default settings for a particular column type. */
 struct column_type_defaults
 {
   unsigned int width;
   bool expand, shrink;
 };
-// Defines the default values for a given type
 
+/** Abstraction of the parameters passed into the layout process. */
+class column_parameters
+{
+public:
+  virtual int param_count()=0;
+  virtual std::wstring get_param(int n)=0;
+
+  virtual ~column_parameters();
+};
+
+/** An empty list of parameters. */
+class empty_column_parameters : public column_parameters
+{
+public:
+  int param_count();
+  std::wstring get_param(int n);
+};
+
+/** Defines how a single column is to be generated. */
 struct column_definition
 {
-  int type;
+  // A literal column is taken from a literal string passed as a
+  // parameter.  A generated one is created by calling a virtual
+  // method.  A parameterized column is taken from a column_parameters
+  // object passed into the layout method.
+  enum column_type {COLUMN_LITERAL, COLUMN_GENERATED, COLUMN_PARAM};
+  column_type type;
+
+  // For parametric columns, this is the parameter number;
+  // for generated ones, it's the type of column:
+  int ival;
+
+  // For literal columns only:
+  std::wstring arg;
+
+  // For generated or parametric columns:
   unsigned int width;
-  bool expand, shrink;
+  bool expand:1, shrink:1;
 
-  std::string arg; // For plain-text columns, this is the column contents
+  // For generated or parametric columns.  If \b true, "width"
+  // will be ignored and the true width of the incoming string will
+  // be given to the layout algorithm.
+  bool dynamic_size:1;
 
-  column_definition(int _type, int _width, bool _expand, bool _shrink,
-		    std::string _arg="")
-    :type(_type), width(_width), expand(_expand), shrink(_shrink), arg(_arg)
+  column_definition(const std::wstring &_arg, bool _expand, bool _shrink)
+    :type(COLUMN_LITERAL), arg(_arg), expand(_expand), shrink(_shrink)
+  {
+  }
+
+  column_definition(column_type _type,
+		    int _ival, int _width, bool _expand, bool _shrink,
+		    bool _dynamic_size)
+    :type(_type), ival(_ival), width(_width),
+     expand(_expand), shrink(_shrink), dynamic_size(_dynamic_size)
   {
     assert(_width>=0);
   }
@@ -85,12 +128,12 @@ public:
 
   virtual ~column_generator();
 
-  std::string layout_columns(unsigned int width);
-  // Lays out the columns into, well, columns.  startlocs is a list of where
-  // the definable groups start -- it should have ngroups entries.
+  std::wstring layout_columns(unsigned int width,
+			      column_parameters &p);
+  // Lays out the columns into columns.
 };
 
-column_definition_list *parse_columns(std::string config,
+column_definition_list *parse_columns(std::wstring config,
 				      column_parser_func parser,
 				      column_type_defaults *defaults);
 // Allocates the array itself; the caller must delete[] it.
