@@ -1,6 +1,6 @@
 // curses++.cc
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 //  A few initialization routines and so on.
 
 #include "curses++.h"
+#include "config/style.h"
 
 #include <stdarg.h>
 
@@ -42,29 +43,59 @@ chstring::chstring(const string &s)
   (*this)=s;
 }
 
-chstring::chstring(const string &s, int attr)
+chstring::chstring(const string &s, const style &st)
 {
   (*this)=s;
-  set_attr(attr);
+  apply_style(st);
 }
 
-// Routines to manipulate chstrings:
-void chstring::add_attr(int attrs)
+chstring::chstring(const chstring &s, const style &st)
+  :super(s)
 {
-  for(chstring::iterator i=begin(); i!=end(); ++i)
-    (*i)|=attrs;
+  (*this)=s;
+  apply_style(st);
 }
 
-void chstring::remove_attr(int attrs)
+void chstring::apply_style(const style &st)
 {
-  for(chstring::iterator i=begin(); i!=end(); ++i)
-    (*i)&=~attrs;
+  for(iterator i=begin(); i!=end(); ++i)
+    *i=st.apply_to(*i);
 }
 
-void chstring::set_attr(int attrs)
+wchstring::wchstring(const wstring &s)
 {
-  for(chstring::iterator i=begin(); i!=end(); ++i)
-    (*i)=((*i)&A_CHARTEXT)|attrs;
+  for(wstring::const_iterator i=s.begin(); i!=s.end(); ++i)
+    push_back(wchtype(*i, A_NORMAL));
+}
+
+wchstring::wchstring(const wstring &s, const style &st)
+{
+  attr_t attrs=st.get_attrs();
+
+  for(wstring::const_iterator i=s.begin(); i!=s.end(); ++i)
+    push_back(wchtype(*i, attrs));
+}
+
+wchstring::wchstring(const wchstring &s, const style &st)
+  :super(s)
+{
+  (*this)=s;
+  apply_style(st);
+}
+
+void wchstring::apply_style(const style &st)
+{
+  for(iterator i=begin(); i!=end(); ++i)
+    *i=st.apply_to(*i);
+}
+
+int wchstring::width() const
+{
+  int rval=0;
+  for(const_iterator i=begin(); i!=end(); ++i)
+    rval+=wcwidth(i->ch);
+
+  return rval;
 }
 
 int char_traits<chtype>::compare(const chtype *s1,
@@ -118,6 +149,57 @@ chtype *char_traits<chtype>::assign(char_type *s,
   return s;
 }
 
+int char_traits<wchtype>::compare(const wchtype *s1,
+				  const wchtype *s2,
+				  size_t n)
+{
+  const wchtype *s1end=s1+n;
+
+  while(s1!=s1end)
+    {
+      wchtype c1=*s1;
+      wchtype c2=*s2;
+
+      if(c1<c2)
+	return -1;
+      else if(c1>c2)
+	return 1;
+
+      ++s1;
+      ++s2;
+    }
+
+  return 0;
+}
+
+size_t char_traits<wchtype>::length (const char_type* s)
+{
+  size_t rval=0;
+
+  while(*s!=eos())
+    {
+      ++rval;
+      ++s;
+    }
+
+  return rval;
+}
+
+wchtype *char_traits<wchtype>::assign(char_type *s,
+				      size_t n,
+				      const char_type &c)
+{
+  char_type *ends=s+n;
+
+  while(s!=ends)
+    {
+      *s=c;
+      ++s;
+    }
+
+  return s;
+}
+
 chstring &chstring::operator=(const std::string &s)
 {
   erase();
@@ -141,6 +223,7 @@ void init_curses()
   keypad(stdscr,TRUE);
 
   start_color();
+  init_colors();
 }
 
 void resize()
@@ -152,7 +235,7 @@ void resize()
       struct winsize w;
       if(ioctl(fd, TIOCGWINSZ, &w)!=-1)
 	{
-	  resizeterm(w.ws_row,w.ws_col);
+	  resize_term(w.ws_row,w.ws_col);
 	  rootwin=newwin(w.ws_row, w.ws_col, 0, 0);
 	  assert(rootwin);
 	  //assert(rootwin.getmaxy()==w.ws_row);
@@ -185,7 +268,9 @@ int cwindow::printw(char *str, ...)
   return amt;
 }
 
-void cwindow::show_string_as_progbar(int x, int y, string s, int attr1, int attr2, int size1, int totalsize)
+void cwindow::show_string_as_progbar(int x, int y, const wstring &s,
+				     int attr1, int attr2, int size1,
+				     int totalsize)
 {
   int width,height;
 
@@ -194,46 +279,133 @@ void cwindow::show_string_as_progbar(int x, int y, string s, int attr1, int attr
   attrset(attr1);
   move(y, x);
 
-  for(int i=0; i<totalsize; i++)
+  size_t loc=0;
+  while(x<width)
     {
-      if(i==size1)
+      if(x>=size1)
 	attrset(attr2);
 
-      if(((unsigned int) i)<s.size())
-	addch((unsigned char) s[i]);
-      // Ack, that's ugly.  Remind me to use strings that have unsigned chars
-      // as their contents next time..
-      else
-	addch(' ');
+      wchar_t ch=L' ';
+      if(loc<s.size())
+	{
+	  ch=s[loc];
+	  ++loc;
+	}
+
+      add_wch(ch);
+      x+=wcwidth(ch);
     }
 }
 
-void cwindow::display_header(string s, int attr)
-  // Y'know, this could be just a call to show_as_progbar or whatever I called
-  // it..
+void cwindow::display_header(wstring s, const attr_t attr)
 {
   attrset(attr);
 
   int width,height;
   getmaxyx(height,width);
 
-  move(0,0);
-  for(int i=0; i<width; i++)
-    addch(' ');
+  move(0, 0);
+  int x=0;
 
-  mvaddnstr(0,0,s.c_str(),width);
+  wstring::size_type sloc=0;
+
+  while(x<width)
+    {
+      if(sloc<s.size())
+	{
+	  const wchar_t wch=s[sloc];
+
+	  add_wch(wch);
+	  x+=wcwidth(wch);
+	  ++sloc;
+	}
+      else
+	{
+	  add_wch(L' ');
+	  x+=wcwidth(L' ');
+	}
+    }
 }
 
-void cwindow::display_status(string s, int attr)
+void cwindow::display_status(wstring s, attr_t attr)
 {
   attrset(attr);
 
   int width,height;
   getmaxyx(height,width);
 
-  move(height-1,0);
-  for(int i=0; i<width; i++)
-    addch(' ');
+  move(height-1, 0);
+  int x=0;
 
-  mvaddnstr(height-1,0,s.c_str(),width);
+  wstring::size_type sloc=0;
+
+  while(x<width)
+    {
+      if(sloc<s.size())
+	{
+	  const wchar_t wch=s[sloc];
+
+	  add_wch(wch);
+	  x+=wcwidth(wch);
+	  ++sloc;
+	}
+      else
+	{
+	  add_wch(L' ');
+	  x+=wcwidth(L' ');
+	}
+    }
+}
+
+int cwindow::addstr(const wchstring &str)
+{
+  return addnstr(str, str.size());
+}
+
+int cwindow::addnstr(const wchstring &str, size_t n)
+{
+  int rval=OK;
+
+  for(string::size_type i=0; i<n && i<str.size(); ++i)
+    {
+      // Construct a cchar_t from the single character at the head of
+      // the string.  The weird intermediate single-character string
+      // exists to work around the vagueness of the setcchar
+      // semantics.
+
+
+      cchar_t wch;
+      wchar_t dummy[2];
+
+      dummy[0]=str[i].ch;
+      dummy[1]=L'\0';
+
+      // How can I notify the user of errors?
+      if(setcchar(&wch, dummy, str[i].attrs,
+		  PAIR_NUMBER(str[i].attrs), 0) == ERR)
+	{
+	  rval=ERR;
+	  attr_t a=get_style("Error").get_attrs();
+	  if(setcchar(&wch, L"?", a, PAIR_NUMBER(a), 0) == ERR)
+	    continue;
+	}
+
+      if(wadd_wch(win, &wch) == ERR)
+	rval=ERR;
+    }
+
+  return rval;
+}
+
+int cwindow::mvaddstr(int y, int x, const wchstring &str)
+{
+  return mvaddnstr(y, x, str, str.size());
+}
+
+int cwindow::mvaddnstr(int y, int x, const wchstring &str, size_t n)
+{
+  if(move(y, x) == ERR)
+    return ERR;
+  else
+    return addnstr(str, n);
 }
