@@ -6,12 +6,16 @@
 
 int vs_scrollbar::get_slider()
 {
+  vs_widget_ref tmpref(this);
+
   int width = dir==HORIZONTAL?getmaxx():getmaxy();
   return max==0?-1:(width-1)*val/max;
 }
 
-void vs_scrollbar::paint()
+void vs_scrollbar::paint(const style &st)
 {
+  vs_widget_ref tmpref(this);
+
   if(dir==HORIZONTAL)
     {
       int width=getmaxx();
@@ -19,9 +23,9 @@ void vs_scrollbar::paint()
 
       for(int x=0; x<width; x++)
 	if(x==thumbloc)
-	  mvaddch(0, x, '#');
+	  mvadd_wch(0, x, L'#');
 	else
-	  mvaddch(0, x, ACS_CKBOARD);
+	  mvadd_wch(0, x, WACS_CKBOARD);
     }
   else
     {
@@ -30,15 +34,20 @@ void vs_scrollbar::paint()
 
       for(int y=0; y<height; y++)
 	if(y==thumbloc)
-	  mvaddch(y, 0, '#');
+	  mvadd_wch(y, 0, L'#');
 	else
-	  mvaddch(y, 0, ACS_CKBOARD);
+	  mvadd_wch(y, 0, WACS_CKBOARD);
     }
 }
 
-size vs_scrollbar::size_request()
+int vs_scrollbar::width_request()
 {
-  return size(1, 1);
+  return 1;
+}
+
+int vs_scrollbar::height_request(int w)
+{
+  return 1;
 }
 
 void vs_scrollbar::set_slider(int newval, int newmax)
@@ -66,6 +75,8 @@ void vs_scrollbar::dispatch_mouse(short id,
 				  int x, int y, int z,
 				  mmask_t bstate)
 {
+  vs_widget_ref tmpref(this);
+
   int slider_loc=get_slider();
   int mloc = dir==HORIZONTAL?x:y;
 
