@@ -3,19 +3,36 @@
 #include "vscreen.h"
 #include "vs_stacked.h"
 
-#include <sigc++/bind.h>
-#include <sigc++/object_slot.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/functors/mem_fun.h>
 
 vs_stacked::vs_stacked(int w, int h)
   :req_w(w), req_h(h)
 {
-  do_layout.connect(slot(*this, &vs_stacked::layout_me));
+  do_layout.connect(sigc::mem_fun(*this, &vs_stacked::layout_me));
 }
 
-void vs_stacked::add_widget(vscreen_widget *w)
+vs_stacked::~vs_stacked()
 {
-  SigC::Connection shown_conn=w->shown_sig.connect(bind(slot(*this, &vs_stacked::raise_widget), w));
-  SigC::Connection hidden_conn=w->hidden_sig.connect(slot(*this, &vs_stacked::hide_widget));
+  assert(children.empty());
+}
+
+void vs_stacked::destroy()
+{
+  vs_widget_ref tmpref(this);
+
+  while(!children.empty())
+    children.front().w->destroy();
+
+  vs_passthrough::destroy();
+}
+
+void vs_stacked::add_widget(const vs_widget_ref &w)
+{
+  vs_widget_ref tmpref(this);
+
+  sigc::connection shown_conn=w->shown_sig.connect(sigc::bind(sigc::mem_fun(*this, &vs_stacked::raise_widget_bare), w.weak_ref()));
+  sigc::connection hidden_conn=w->hidden_sig.connect(sigc::mem_fun(*this, &vs_stacked::hide_widget));
 
   defocus();
 
@@ -34,8 +51,12 @@ void vs_stacked::hide_widget()
   vscreen_update();
 }
 
-void vs_stacked::rem_widget(vscreen_widget *w)
+void vs_stacked::rem_widget(const vs_widget_ref &wBare)
 {
+  vs_widget_ref tmpref(this);
+
+  vs_widget_ref w(wBare);
+
   for(childlist::iterator i=children.begin();
       i!=children.end();
       i++)
@@ -58,8 +79,10 @@ void vs_stacked::rem_widget(vscreen_widget *w)
     }
 }
 
-void vs_stacked::raise_widget(vscreen_widget *w)
+void vs_stacked::raise_widget(const vs_widget_ref &w)
 {
+  vs_widget_ref tmpref(this);
+
   for(childlist::iterator i=children.begin();
       i!=children.end();
       i++)
@@ -77,8 +100,10 @@ void vs_stacked::raise_widget(vscreen_widget *w)
       }
 }
 
-void vs_stacked::lower_widget(vscreen_widget *w)
+void vs_stacked::lower_widget(const vs_widget_ref &w)
 {
+  vs_widget_ref tmpref(this);
+
   for(childlist::iterator i=children.begin();
       i!=children.end();
       i++)
@@ -96,18 +121,22 @@ void vs_stacked::lower_widget(vscreen_widget *w)
       }
 }
 
-void vs_stacked::paint()
+void vs_stacked::paint(const style &st)
 {
+  vs_widget_ref tmpref(this);
+
   // Go through the children back-to-front (reverse order)
   for(childlist::reverse_iterator i=children.rbegin();
       i!=children.rend();
       i++)
     if(i->w->get_visible())
-      i->w->display();
+      i->w->display(st);
 }
 
 void vs_stacked::dispatch_mouse(short id, int x, int y, int z, mmask_t bstate)
 {
+  vs_widget_ref tmpref(this);
+
   for(childlist::iterator i=children.begin();
       i!=children.end();
       ++i)
@@ -119,8 +148,10 @@ void vs_stacked::dispatch_mouse(short id, int x, int y, int z, mmask_t bstate)
       }
 }
 
-vscreen_widget *vs_stacked::get_focus()
+vs_widget_ref vs_stacked::get_focus()
 {
+  vs_widget_ref tmpref(this);
+
   for(childlist::iterator i=children.begin();
       i!=children.end();
       i++)
@@ -134,6 +165,8 @@ vscreen_widget *vs_stacked::get_focus()
 
 void vs_stacked::show_all()
 {
+  vs_widget_ref tmpref(this);
+
   defocus();
 
   for(childlist::iterator i=children.begin();
@@ -144,19 +177,26 @@ void vs_stacked::show_all()
 
       i->w->show_all();
 
-      i->shown_conn=i->w->shown_sig.connect(bind(slot(*this, &vs_stacked::raise_widget), i->w));
+      i->shown_conn=i->w->shown_sig.connect(sigc::bind(sigc::mem_fun(*this, &vs_stacked::raise_widget_bare), i->w.weak_ref()));
     }
 
   refocus();
 }
 
-size vs_stacked::size_request()
+int vs_stacked::width_request()
 {
-  return size(req_w, req_h);
+  return req_w;
+}
+
+int vs_stacked::height_request(int w)
+{
+  return req_h;
 }
 
 void vs_stacked::layout_me()
 {
+  vs_widget_ref tmpref(this);
+
   for(childlist::iterator i=children.begin(); i!=children.end(); i++)
     i->w->alloc_size(0, 0, getmaxx(), getmaxy());
 }
