@@ -1,6 +1,6 @@
 // vs_subtree.h  (this is -*-c++-*-)
 //
-//  Copyright 1999 Daniel Burrows
+//  Copyright 1999-2003, 2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 //  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 //  Boston, MA 02111-1307, USA.
 //
-//  Subtrees for vs_trees  [ my header files are getting out of hand :P ]
+//  Subtrees for vs_trees.
 
 #ifndef VS_SUBTREE_H
 #define VS_SUBTREE_H
@@ -96,36 +96,55 @@ public:
   }
 
   void paint(vs_tree *win, int y, bool hierarchical,
-	     std::string str, int depth_shift=2)
+	     const std::wstring &str, int depth_shift=2)
   {
     int width, height;
     int basex=hierarchical?depth_shift*get_depth():0;
     win->getmaxyx(height,width);
 
     win->move(y,0);
-    for(int i=0; i<basex && i<width; i++)
-      win->addch(' ');
+
+    int x=0;
+    while(x<basex && x<width)
+      {
+	win->add_wch(L' ');
+	x+=wcwidth(L' ');
+      }
 
     if(basex>width)
       return;
 
+    const wchar_t *ws;
     if(hierarchical)
-      win->addnstr(get_expanded()?"--\\ ":"--- ", width-basex);
+      ws=get_expanded()?L"--\\ ":L"--- ";
     else
-      win->addnstr("-> ", width-basex);
+      ws=L"-> ";
 
-    if(basex+(hierarchical?4:3)>width)
+    while(*ws!=0 && x<width)
+      {
+	win->add_wch(*ws);
+	x+=wcwidth(*ws);
+	++ws;
+      }
+
+    if(x>=width)
       return;
 
-    win->addnstr(str.c_str(), width-(basex+(hierarchical?4:3)));
+    size_t i=0;
+    while(i<str.size() && x<width)
+      {
+	wchar_t ch=str[i];
 
-    int newy,newx;
-    win->getyx(newy,newx);
+	win->add_wch(ch);
+	x+=wcwidth(ch);
+	++i;
+      }
 
-    if(newy==y)
-      // Eek!  If we overflow on that last addnstr, newy will advance!!
-      for( ; newx<width; newx++)
-	win->addch(' ');
+    while(x<width)
+      {
+	win->add_wch(L' ');
+	x+=wcwidth(L' ');
+      }
   }
 
   void set_depth(int _depth)
@@ -146,7 +165,7 @@ public:
   // Adds a new child item at an unspecified location -- you should call sort()
   // after adding children or the tree will have an undetermined order.  (yes,
   // you can deduce the real order.  Don't.)
-  void sort(const sortpolicy &sort_method)
+  void sort(sortpolicy &sort_method)
   {
     for(child_iterator i=children.begin(); i!=children.end(); i++)
       (*i)->sort(sort_method);
@@ -160,23 +179,24 @@ public:
     sort(sorter);
   }
 
-  virtual bool dispatch_char(chtype ch, vs_tree *owner)
+  virtual bool dispatch_key(const key &k, vs_tree *owner)
   {
-    if(vs_tree::bindings->key_matches(ch, "ToggleExpanded"))
+    if(vs_tree::bindings->key_matches(k, "ToggleExpanded"))
       {
 	expanded=!expanded;
 	return true;
       }
-    else if(vs_tree::bindings->key_matches(ch, "ExpandTree"))
+    else if(vs_tree::bindings->key_matches(k, "ExpandTree"))
       {
         if(!expanded)
-	{
-	  expanded=true;
-	  return true;
-	} else
+	  {
+	    expanded=true;
+	    return true;
+	  }
+	else
 	  return false;
       }
-    else if(vs_tree::bindings->key_matches(ch, "CollapseTree"))
+    else if(vs_tree::bindings->key_matches(k, "CollapseTree"))
       {
 	if(expanded)
 	{
@@ -185,12 +205,12 @@ public:
 	} else
 	  return false;
       }
-    else if(vs_tree::bindings->key_matches(ch, "ExpandAll"))
+    else if(vs_tree::bindings->key_matches(k, "ExpandAll"))
       {
 	expand_all();
 	return true;
       }
-    else if(vs_tree::bindings->key_matches(ch, "CollapseAll"))
+    else if(vs_tree::bindings->key_matches(k, "CollapseAll"))
       {
 	collapse_all();
 	return true;
