@@ -1,6 +1,6 @@
 // vs_tree.cc
 //
-//  Copyright 1999 Daniel Burrows
+//  Copyright 1999-2002, 2004-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,11 @@
 #include "vs_editline.h"
 #include "config/keybindings.h"
 #include "config/colors.h"
+#include "transcode.h"
+
+#include "../aptitude.h" // For _()
+
+#include <sigc++/functors/ptr_fun.h>
 
 using namespace std;
 
@@ -43,8 +48,8 @@ vs_tree::vs_tree()
    hierarchical(true),
    prev_level(NULL)
 {
-  focussed.connect(SigC::slot(vscreen_update));
-  unfocussed.connect(SigC::slot(vscreen_update));
+  focussed.connect(sigc::ptr_fun(vscreen_update));
+  unfocussed.connect(sigc::ptr_fun(vscreen_update));
 }
 
 vs_tree::vs_tree(vs_treeitem *_root, bool showroot)
@@ -59,8 +64,8 @@ vs_tree::vs_tree(vs_treeitem *_root, bool showroot)
 {
   set_root(_root, showroot);
 
-  focussed.connect(SigC::slot(vscreen_update));
-  unfocussed.connect(SigC::slot(vscreen_update));
+  focussed.connect(sigc::ptr_fun(vscreen_update));
+  unfocussed.connect(sigc::ptr_fun(vscreen_update));
 }
 
 vs_tree::~vs_tree()
@@ -81,9 +86,14 @@ void vs_tree::do_shown()
     selected->highlighted(this);
 }
 
-size vs_tree::size_request()
+int vs_tree::width_request()
 {
-  return size(1, 1);
+  return 1;
+}
+
+int vs_tree::height_request(int w)
+{
+  return 1;
 }
 
 void vs_tree::set_root(vs_treeitem *_root, bool showroot)
@@ -185,7 +195,8 @@ int vs_tree::line_of(vs_treeiterator item)
       return j;
   } while(i!=begin);
 
-  assert(0);
+  // Only happens if the iterator isn't in the visible range at all.
+  abort();
 }
 
 bool vs_tree::item_visible(vs_treeiterator pkg)
@@ -209,6 +220,18 @@ bool vs_tree::item_visible(vs_treeiterator pkg)
 
 void vs_tree::set_selection(vs_treeiterator to)
 {
+  // Expand all its parents so that it's possible to make it visible.
+  vs_treeiterator curr = to;
+  while(!curr.is_root())
+    {
+      curr = curr.get_up();
+      curr.expand();
+    }
+
+  // Expand the root as well if necessary.
+  if(curr != to)
+    curr.expand();
+
   if(item_visible(to))
     {
       if(selected!=end)
@@ -221,11 +244,50 @@ void vs_tree::set_selection(vs_treeiterator to)
     }
   else
     {
-      if(selected!=end)
-	selected->unhighlighted(this);
-      selected=top=to;
-      if(selected!=end)
-	selected->highlighted(this);
+      int height = getmaxy();
+      if(height == 0)
+	{
+	  selected = top = to;
+	  return;
+	}
+
+      // Give up and just directly determine the line of 'to'.
+      int l = line_of(to);
+
+      while(l < 1)
+	{
+	  assert(top != end);
+
+	  if(hierarchical)
+	    --top;
+	  else
+	    top.move_backward_level();
+
+	  ++l;
+	}
+
+      while(l > height)
+	{
+	  assert(top != end);
+
+	  if(hierarchical)
+	    ++top;
+	  else
+	    top.move_forward_level();
+
+	  --l;
+	}
+
+      if(selected != to)
+	{
+	  if(selected != end)
+	    selected->unhighlighted(this);
+
+	  if(to != end)
+	    to->highlighted(this);
+	}
+
+      selected = to;
 
       vscreen_update();
     }
@@ -233,12 +295,14 @@ void vs_tree::set_selection(vs_treeiterator to)
 
 bool vs_tree::get_cursorvisible()
 {
-  return (selected!=end && selected->get_selectable());
+  return (root != NULL && selected != end && selected->get_selectable());
 }
 
 point vs_tree::get_cursorloc()
 {
-  if(selected==end || !selected->get_selectable())
+  if(root == NULL)
+    return point(0, 0);
+  else if(selected==end || !selected->get_selectable())
     return point(0,0);
   else
     return point(0, hierarchical?line_of(selected)-1:line_of(selected));
@@ -246,75 +310,67 @@ point vs_tree::get_cursorloc()
 
 void vs_tree::line_down()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
   if(!hierarchical)
     --height;
 
-  if(selected!=end)
+  vs_treeiterator orig = selected, prevtop = top;
+
+  int newline = line_of(selected);
+  int scrollcount = 0;
+  bool moved = false;
+
+  while(selected != end &&
+	scrollcount < 1 &&
+	(!moved || !selected->get_selectable()))
     {
-      vs_treeiterator orig=selected,prevtop=top;
-      selected->unhighlighted(this);
+      if(hierarchical)
+	++selected;
+      else
+	selected.move_forward_level();
 
-      int newline=line_of(selected);
-      bool movedonce=false;
+      ++newline;
+      moved = true;
 
-      while(newline<height &&
-	    selected!=end &&
-	    (!movedonce || !selected->get_selectable()))
+      // If we fell off the end of the screen and not off the end of
+      // the list, scroll the screen forward.
+      if(newline > height && selected != end)
 	{
-	  ++selected;
-	  ++newline;
-	  movedonce=true;
+	  if(hierarchical)
+	    ++top;
+	  else
+	    top.move_forward_level();
+
+	  --newline;
+	  ++scrollcount;
 	}
-
-      if(newline==height &&
-	 selected!=end &&
-	 (!movedonce || !selected->get_selectable()))
-	// We ran off the screen and couldn't find anything to select,
-	// or we started at the bottom of the screen.
-	{
-	  ++selected;
-
-	  if(selected!=end)
-	    {
-	      // Shift the top forward.
-	      ++top;
-
-	      if(!selected->get_selectable())
-		// Ok, well, we still can't select this.  Too bad.
-		{
-		  selected=orig;
-
-		  if(selected==prevtop)
-		    // We moved forward by one, so the previous top
-		    // needs to be moved forward.  (saves an
-		    // expensive line_of call)
-		    {
-		      selected=top;
-		      int l=0;
-
-		      while(selected!=end &&
-			    !selected->get_selectable() &&
-			    l<height)
-			{
-			  ++selected;
-			  ++l;
-			}
-		    }
-		}
-	    }
-	}
-
-      if(selected==end)
-	--selected;
-
-      if(selected!=end)
-	selected->highlighted(this);
-
-      vscreen_update();
     }
+
+  if(selected == end)
+    {
+      if(hierarchical)
+	--selected;
+      else
+	selected.move_backward_level();
+
+      --newline;
+    }
+
+  if(orig != selected)
+    {
+      if(orig != end)
+	orig->unhighlighted(this);
+
+      if(selected != end)
+	selected->highlighted(this);
+    }
+
+  vscreen_update();
 }
 
 void vs_tree::set_hierarchical(bool _hierarchical)
@@ -348,8 +404,23 @@ void vs_tree::set_hierarchical(bool _hierarchical)
     }
 }
 
+void vs_tree::highlight_current()
+{
+  if(root != NULL && selected != end)
+    selected->highlighted(this);
+}
+
+void vs_tree::unhighlight_current()
+{
+  if(root != NULL && selected != end)
+    selected->unhighlighted(this);
+}
+
 void vs_tree::line_up()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
@@ -357,74 +428,59 @@ void vs_tree::line_up()
     --height;
 
   vs_treeiterator orig=selected;
-  if(selected!=end)
-    selected->unhighlighted(this);
 
-  if(selected!=begin)
+  bool moved = false;
+  int scrollcount = 0;
+
+  while(selected != begin &&
+	scrollcount < 1 &&
+	(!moved || !selected->get_selectable()))
     {
-      bool movedonce=false;
-
-      while(selected!=begin &&
-	    selected!=top &&
-	    (!movedonce || !selected->get_selectable()))
+      if(selected == top)
 	{
-	  if(hierarchical)
-	    --selected;
-	  else
-	    selected.move_backward_level();
-	  movedonce=true;
-	}
-
-      if(selected==top &&
-	 selected!=begin &&
-	 (!movedonce || !selected->get_selectable()))
-	// We ran off the screen and couldn't find anything to select,
-	// or we started at the top of the screen.
-	{
-	  if(hierarchical)
-	    --selected;
-	  else
-	    selected.move_backward_level();
-
-	  // The top always moves.  If it is the beginning, something
-	  // is seriously messed up.
-	  assert(top!=begin);
 	  if(hierarchical)
 	    --top;
 	  else
 	    top.move_backward_level();
-	  if(!selected->get_selectable())
-	    // Oops.  Even moving one more doesn't give us anything
-	    // to select.  Restore the selection.
-	    {
-	      selected=orig;
-	      if(line_of(selected)>height)
-		// If the selection was pushed off the screen,
-		// bring it back on.
-		{
-		  if(hierarchical)
-		    --selected;
-		  else
-		    selected.move_backward_level();
 
-		  while(selected!=top && !selected->get_selectable())
-		    if(hierarchical)
-		      --selected;
-		    else
-		      selected.move_backward_level();
-		}
-	    }
+	  ++scrollcount;
 	}
+
+      if(hierarchical)
+	--selected;
+      else
+	selected.move_backward_level();
+      moved = true;
     }
 
-  if(selected!=end)
-    selected->highlighted(this);
+  // Handle the special case where the first element of the tree is
+  // non-selectable.
+  if(selected == begin && !selected->get_selectable())
+    {
+      while(selected != end && !selected->get_selectable())
+	++selected;
+
+      if(line_of(selected) >= height)
+	selected = begin;
+    }
+
+  if(orig != selected)
+    {
+      if(orig != end)
+	orig->unhighlighted(this);
+
+      if(selected != end)
+	selected->highlighted(this);
+    }
 
   vscreen_update();
 }
 
 void vs_tree::page_down()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
@@ -461,6 +517,9 @@ void vs_tree::page_down()
 
 void vs_tree::page_up()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
@@ -493,13 +552,16 @@ void vs_tree::page_up()
 	selected=top;
 
       if(selected!=end)
-	(*selected).unhighlighted(this);
+	(*selected).highlighted(this);
       vscreen_update();
     }
 }
 
 void vs_tree::jump_to_begin()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
@@ -532,6 +594,9 @@ void vs_tree::jump_to_begin()
 
 void vs_tree::jump_to_end()
 {
+  if(root == NULL)
+    return;
+
   int width,height;
   getmaxyx(height,width);
 
@@ -580,6 +645,9 @@ void vs_tree::jump_to_end()
 
 void vs_tree::level_line_up()
 {
+  if(root == NULL)
+    return;
+
   vs_treeiterator tmp=selected;
   tmp.move_backward_level();
   if(tmp!=end)
@@ -588,13 +656,16 @@ void vs_tree::level_line_up()
 
 void vs_tree::level_line_down()
 {
+  if(root == NULL)
+    return;
+
   vs_treeiterator tmp=selected;
   tmp.move_forward_level();
   if(tmp!=end)
     set_selection(tmp);
 }
 
-bool vs_tree::handle_char(chtype ch)
+bool vs_tree::handle_key(const key &k)
 {
   // umm...
   //width++;
@@ -602,12 +673,12 @@ bool vs_tree::handle_char(chtype ch)
 
   if(selected!=vs_treeiterator(NULL))
     {
-      if(hierarchical && bindings->key_matches(ch, "Parent"))
+      if(root != NULL && hierarchical && bindings->key_matches(k, "Parent"))
 	{
 	  if(!selected.is_root())
 	    set_selection(selected.get_up());
 	}
-      else if(!hierarchical && prev_level && bindings->key_matches(ch, "Left"))
+      else if(root != NULL && !hierarchical && prev_level && bindings->key_matches(k, "Left"))
 	{
 	  selected->unhighlighted(this);
 
@@ -624,11 +695,11 @@ bool vs_tree::handle_char(chtype ch)
 
 	  vscreen_update();
 	}
-      else if(!hierarchical &&
+      else if(root != NULL && !hierarchical &&
 	      selected!=end && selected->get_selectable() &&
 	      selected->begin()!=selected->end() &&
-	      (bindings->key_matches(ch, "Right") ||
-	       bindings->key_matches(ch, "Confirm")))
+	      (bindings->key_matches(k, "Right") ||
+	       bindings->key_matches(k, "Confirm")))
 	{
 	  selected->unhighlighted(this);
 	  prev_level=new flat_frame(begin, end, top, selected, prev_level);
@@ -642,26 +713,26 @@ bool vs_tree::handle_char(chtype ch)
 
 	  vscreen_update();
 	}
-      else if(bindings->key_matches(ch, "Down"))
+      else if(bindings->key_matches(k, "Down"))
 	line_down();
-      else if(bindings->key_matches(ch, "Up"))
+      else if(bindings->key_matches(k, "Up"))
 	line_up();
-      else if(bindings->key_matches(ch, "NextPage"))
+      else if(bindings->key_matches(k, "NextPage"))
 	page_down();
-      else if(bindings->key_matches(ch, "PrevPage"))
+      else if(bindings->key_matches(k, "PrevPage"))
 	page_up();
-      else if(bindings->key_matches(ch, "Begin"))
+      else if(bindings->key_matches(k, "Begin"))
 	jump_to_begin();
-      else if(bindings->key_matches(ch, "End"))
+      else if(bindings->key_matches(k, "End"))
 	jump_to_end();
-      else if(bindings->key_matches(ch, "LevelUp"))
+      else if(bindings->key_matches(k, "LevelUp"))
 	level_line_up();
-      else if(bindings->key_matches(ch, "LevelDown"))
+      else if(bindings->key_matches(k, "LevelDown"))
 	level_line_down();
       /*else if(bindings->key_matches(ch, "Search"))
 	{
 	  vs_statusedit *ed=new vs_statusedit("Search for: ");
-	  ed->entered.connect(slot(this, &vs_tree::search_for));
+	  ed->entered.connect(sigc::mem_fun(this, &vs_tree::search_for));
 	  add_widget(ed);
 	  vscreen_update();
 	}
@@ -669,10 +740,11 @@ bool vs_tree::handle_char(chtype ch)
       search_for("");*/
       else
 	{
-	  if(selected!=end && selected->get_selectable() && selected->dispatch_char(ch, this))
+	  if(root != NULL && selected!=end && selected->get_selectable() &&
+	     selected->dispatch_key(k, this))
 	    vscreen_update();
 	  else
-	    return vscreen_widget::handle_char(ch);
+	    return vscreen_widget::handle_key(k);
 	}
       return true;
     }
@@ -681,6 +753,9 @@ bool vs_tree::handle_char(chtype ch)
 
 void vs_tree::search_for(vs_tree_search_func &matches)
 {
+  if(root == NULL)
+    return;
+
   vs_treeiterator curr((selected==vs_treeiterator(NULL))?begin:selected, hierarchical),
     start(curr);
   // Make an iterator that ignores all the rules >=)
@@ -712,20 +787,93 @@ void vs_tree::search_for(vs_tree_search_func &matches)
     beep();
   else
     {
-      selected->unhighlighted(this);
       set_selection(curr);
-      selected->highlighted(this);
-      while(!curr.is_root())
-	{
-	  curr=curr.get_up();
-	  curr.expand();
-	}
       vscreen_update();
     }
 }
 
-void vs_tree::paint()
+void vs_tree::search_back_for(vs_tree_search_func &matches)
 {
+  if(root == NULL)
+    return;
+
+  vs_treeiterator curr((selected == vs_treeiterator(NULL))
+		       ? begin : selected, hierarchical),
+    start(curr);
+
+  // Skip the starting location, cycling to the end.
+  if(curr != begin)
+    {
+      if(hierarchical)
+	--curr;
+      else
+	curr.move_backward_level();
+    }
+  else
+    {
+      if(hierarchical)
+	{
+	  curr = end;
+	  --curr;
+	}
+      else
+	{
+	  vs_treeiterator curr2 = curr;
+	  curr2.move_forward_level();
+
+	  while(curr2 != curr)
+	    {
+	      curr = curr2;
+	      curr2.move_forward_level();
+	    }
+	}
+    }
+
+  while(curr != start && !matches(*curr))
+    {
+      // Code duplication alert
+      if(curr != begin)
+	{
+	  if(hierarchical)
+	    --curr;
+	  else
+	    curr.move_backward_level();
+	}
+      else
+	{
+	  if(hierarchical)
+	    {
+	      curr = end;
+	      --curr;
+	    }
+	  else
+	    {
+	      vs_treeiterator curr2 = curr;
+	      curr2.move_forward_level();
+
+	      while(curr2 != curr)
+		{
+		  curr = curr2;
+		  curr2.move_forward_level();
+		}
+	    }
+	}
+    }
+
+  if(curr == start)
+    beep();
+  else
+    {
+      set_selection(curr);
+      vscreen_update();
+    }
+}
+
+void vs_tree::paint(const style &st)
+{
+  if(root == NULL)
+    return;
+
   int width,height;
   int selectedln=line_of(selected);
 
@@ -754,8 +902,8 @@ void vs_tree::paint()
 	}
     }
 
-  if(selected!=end && selected->get_selectable())
-    selected->highlighted(this);
+  //if(selected!=end && selected->get_selectable())
+  //selected->highlighted(this);
   // Some classes need this to update display stuff properly.  For instance,
   // when a new pkg_tree is created, its 'update the status line' signal
   // won't be properly called without this.
@@ -765,7 +913,7 @@ void vs_tree::paint()
 
   if(!hierarchical && y<height)
     {
-      string todisp="";
+      wstring todisp;
 
       // Uh...I'd rather use the iterators to do this..
       flat_frame *curr=prev_level;
@@ -774,17 +922,17 @@ void vs_tree::paint()
 	  if(todisp.empty())
 	    todisp=curr->selected->label()+todisp;
 	  else
-	    todisp=curr->selected->label()+("::"+todisp);
+	    todisp=curr->selected->label()+(L"::"+todisp);
 	  curr=curr->next;
 	}
 
       if(todisp.empty())
-	todisp="TOP LEVEL";
+	todisp=transcode(_("TOP LEVEL"));
 
       while(todisp.size()<(unsigned) width)
-	todisp+=" ";
+	todisp+=L" ";
 
-      attrset(get_color("ScreenHeaderColor"));
+      apply_style(st+get_style("Header"));
       mvaddnstr(y, 0, todisp.c_str(), width);
 
       ++y;
@@ -797,12 +945,15 @@ void vs_tree::paint()
     {
       vs_treeitem *curr=&*i;
 
-      if(get_isfocussed() && i==selected && i->get_selectable())
-	attrset(curr->get_highlight_attr());
-      else
-	attrset(curr->get_normal_attr());
+      style curr_st;
 
-      curr->paint(this, y, hierarchical);
+      if(get_isfocussed() && i==selected && i->get_selectable())
+	curr_st = st+curr->get_highlight_style();
+      else
+	curr_st = st+curr->get_normal_style();
+
+      apply_style(curr_st);
+      curr->paint(this, y, hierarchical, curr_st);
 
       if(hierarchical)
 	++i;
@@ -819,6 +970,9 @@ void vs_tree::paint()
 
 void vs_tree::dispatch_mouse(short id, int x, int y, int z, mmask_t bstate)
 {
+  if(root == NULL)
+    return;
+
   if(!hierarchical)
     --y;
 
