@@ -6,8 +6,10 @@
 // needed for queuelayout:
 #include "vscreen.h"
 
-#include <sigc++/bind.h>
-#include <sigc++/object_slot.h>
+#include <set>
+
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/functors/mem_fun.h>
 
 #include <config/colors.h>
 #include <config/keybindings.h>
@@ -15,13 +17,12 @@
 #include <assert.h>
 
 #ifdef HAVE_CONFIG_H
-#include "../../config.h"
+#include <config.h>
 #endif
 
 using namespace std;
 
 // Queues of things that Need To Be Done
-static list<vscreen_widget *> todelete;
 static list<vscreen_widget *> toresize;
 
 vscreen_widget::vscreen_widget()
@@ -29,17 +30,31 @@ vscreen_widget::vscreen_widget()
    timeout_value(0),
    owner(NULL),
    geom(0,0,0,0),
+   refcount(1),
    visible(false),
-   bgattr(get_color("DefaultWidgetBackground")),
-   isfocussed(false)
+   isfocussed(false),
+   pre_display_erase(true),
+   is_destroyed(false)
 {
-  focussed.connect(SigC::bind(SigC::slot(*this, &vscreen_widget::set_isfocussed), true));
-  unfocussed.connect(SigC::bind(SigC::slot(*this, &vscreen_widget::set_isfocussed), false));
+  focussed.connect(sigc::bind(sigc::mem_fun(*this, &vscreen_widget::set_isfocussed), true));
+  unfocussed.connect(sigc::bind(sigc::mem_fun(*this, &vscreen_widget::set_isfocussed), false));
 }
 
 vscreen_widget::~vscreen_widget()
 {
   assert(!owner);
+  assert(is_destroyed);
+}
+
+void vscreen_widget::set_bg_style(const style &new_style)
+{
+  bg_style=new_style;
+}
+
+void vscreen_widget::apply_style(const style &st)
+{
+  bkgdset(st.get_attrs());
+  attrset(st.get_attrs());
 }
 
 void vscreen_widget::set_isfocussed(bool _isfocussed)
@@ -47,20 +62,10 @@ void vscreen_widget::set_isfocussed(bool _isfocussed)
   isfocussed=_isfocussed;
 }
 
-void vscreen_widget::handle_pending_deletes()
-{
-  // don't delete -- debugging
-
-  //for(list<vscreen_widget *>::iterator i=todelete.begin();
-  //    i!=todelete.end();
-      //    i++)
-  //  delete *i;
-
-  todelete.erase(todelete.begin(), todelete.end());
-}
-
 void vscreen_widget::set_owner_window(cwindow _win, int x, int y, int w, int h)
 {
+  vs_widget_ref tmpref(this);
+
   if(_win)
     {
       geom.x=x;
@@ -72,6 +77,8 @@ void vscreen_widget::set_owner_window(cwindow _win, int x, int y, int w, int h)
 	win=NULL;
       else
 	{
+	  assert(!is_destroyed);
+
 	  win=_win.derwin(geom.h,
 			  geom.w,
 			  geom.y,
@@ -98,9 +105,6 @@ void vscreen_widget::set_owner(vs_container *_owner)
 {
   owner=_owner;
   alloc_size(0,0,0,0);
-
-  if(owner==NULL)
-    destroy();
 }
 
 int vscreen_widget::timeout(int msecs)
@@ -112,37 +116,54 @@ int vscreen_widget::timeout(int msecs)
   return prevval;
 }
 
+void vscreen_widget::cleanup()
+{
+  assert(is_destroyed);
+  delete this;
+}
+
 void vscreen_widget::destroy()
 {
+  assert(refcount > 0);
+
+  if(is_destroyed)
+    return;
+
+  // Make sure we don't die during the destroy routine.
+  vs_widget_ref this_ref = this;
+
   hide();
 
   if(owner)
-    owner->rem_widget(this);
+    {
+      owner->rem_widget(this);
+      assert(!win);
+    }
 
-  assert(!win);
+  // This must be done after hide() and rem_widget(), because
+  // otherwise some of the vscreen_widget manipulators become NOPs,
+  // which causes confusion and problems for code that accesses them.
+  // For instance, the multiplex rem_widget code expects hide_widget
+  // to do something sensible.  I could try to fix this, but it's much
+  // more straightforward to just leave the widget in a non-destroyed
+  // state until after it's disconnected from everything.
+  is_destroyed = true;
 
   destroyed();
+}
 
-  // Disconnect all signals:
-
-  shown_sig.clear();
-  hidden_sig.clear();
-  destroyed.clear();
-  do_layout.clear();
-  focussed.clear();
-  unfocussed.clear();
-
-  for(list<vscreen_widget *>::iterator i=todelete.begin();
-      i!=todelete.end();
-      i++)
-    if(*i==this)
-      return;
-
-  todelete.push_back(this);
+ref_ptr<vs_container> vscreen_widget::get_owner()
+{
+  return owner;
 }
 
 void vscreen_widget::show()
 {
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return;
+
   visible=true;
 
   shown_sig();
@@ -150,47 +171,69 @@ void vscreen_widget::show()
 
 void vscreen_widget::show_all()
 {
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return;
+
   show();
 }
 
 void vscreen_widget::hide()
 {
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return;
+
   visible=false;
 
   hidden_sig();
 }
 
-void vscreen_widget::display()
+void vscreen_widget::display(const style &st)
 {
-  if(bgattr!=-1)
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return;
+
+  // Erase our window, using the composition of the surrounding style
+  // and our background style.
+  style basic_st=st+bg_style;
+  int bgattr=basic_st.get_attrs();
+
+  if(pre_display_erase)
     {
       bkgd(bgattr);
       erase();
-      attrset(bgattr);
     }
 
-  paint();
-}
-
-void vscreen_widget::set_bg(int _bgattr)
-{
-  // Should I queue a redraw?  Right now I'm not doing that..
-  bgattr=_bgattr;
+  attrset(bgattr);
+  paint(basic_st);
 }
 
 bool vscreen_widget::focus_me()
 {
+  if(is_destroyed)
+    return false;
+
   return !auxillary_bindings.empty();
 }
 
-bool vscreen_widget::handle_char(chtype ch)
+bool vscreen_widget::handle_key(const key &k)
 {
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return false;
+
   bool rval=false;
 
   for(key_connection i=auxillary_post_bindings.begin();
       i!=auxillary_post_bindings.end();
       i++)
-    if(i->bindings->key_matches(ch, i->keyname))
+    if(i->bindings->key_matches(k, i->keyname))
       {
 	i->slot();
 	rval=true;
@@ -199,20 +242,25 @@ bool vscreen_widget::handle_char(chtype ch)
   return rval;
 }
 
-bool vscreen_widget::dispatch_char(chtype ch)
+bool vscreen_widget::dispatch_key(const key &k)
 {
+  vs_widget_ref tmpref(this);
+
+  if(is_destroyed)
+    return false;
+
   bool rval=false;
 
   for(key_connection i=auxillary_bindings.begin();
       i!=auxillary_bindings.end();
       i++)
-    if(i->bindings->key_matches(ch, i->keyname))
+    if(i->bindings->key_matches(k, i->keyname))
       {
 	i->slot();
 	rval=true;
       }
 
-  return rval || handle_char(ch);
+  return rval || handle_key(k);
 }
 
 void vscreen_widget::dispatch_mouse(short id, int x, int y, int z,
@@ -222,7 +270,7 @@ void vscreen_widget::dispatch_mouse(short id, int x, int y, int z,
 
 vscreen_widget::key_connection vscreen_widget::connect_key(const string &key,
 							   keybindings *bindings,
-							   const SigC::Slot0<void> &slot)
+							   const sigc::slot0<void> &slot)
 {
   auxillary_bindings.push_back(binding_connection(key, bindings, slot));
 
@@ -234,7 +282,7 @@ vscreen_widget::key_connection vscreen_widget::connect_key(const string &key,
 
 vscreen_widget::key_connection vscreen_widget::connect_key_post(const string &key,
 								keybindings *bindings,
-								const SigC::Slot0<void> &slot)
+								const sigc::slot0<void> &slot)
 {
   auxillary_post_bindings.push_back(binding_connection(key, bindings, slot));
 
